@@ -1,6 +1,6 @@
 # important_changes.pl - This script prints important changes in Sympa since last install
 # It is based on the NEWS ***** entries
-# RCS Identication ; $Revision: 1.5 $ ; $Date: 2002/01/25 09:46:17 $ 
+# RCS Identication ; $Revision: 1.5.2.2 $ ; $Date: 2003/04/16 11:29:51 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -48,7 +48,8 @@ unless ($first_install) {
 }
 close VERSION;
 
-if ($previous_version eq $current_version) {
+if (($previous_version eq $current_version) ||
+    &higher($previous_version,$current_version)){
     exit 0;
 }
 
@@ -80,3 +81,38 @@ close NOTES;
 print "<RETURN>";
 my $wait = <STDIN>;
 
+sub higher {
+    my ($v1, $v2) = @_;
+
+    my @tab1 = split /\./,$v1;
+    my @tab2 = split /\./,$v2;
+    
+    
+    my $max = $#tab1;
+    $max = $#tab2 if ($#tab2 > $#tab1);
+
+    for $i (0..$max) {
+    
+        if ($tab1[0] =~ /^(\d*)a$/) {
+            $tab1[0] = $1 - 0.5;
+        }elsif ($tab1[0] =~ /^(\d*)b$/) {
+            $tab1[0] = $1 - 0.25;
+        }
+
+        if ($tab2[0] =~ /^(\d*)a$/) {
+            $tab2[0] = $1 - 0.5;
+        }elsif ($tab2[0] =~ /^(\d*)b$/) {
+            $tab2[0] = $1 - 0.25;
+        }
+
+        if ($tab1[0] eq $tab2[0]) {
+            #printf "\t%s = %s\n",$tab1[0],$tab2[0];
+            shift @tab1;
+            shift @tab2;
+            next;
+        }
+        return ($tab1[0] > $tab2[0]);
+    }
+
+    return 0;
+}
