@@ -1,5 +1,5 @@
-# Commands.pm - this module does the mail commands processing
-# RCS Identication ; $Revision: 1.62 $ ; $Date: 2002/01/22 11:26:35 $ 
+# Command.pm - this module does the mail commands processing
+# RCS Identication ; $Revision: 1.79.2.3 $ ; $Date: 2003/07/17 07:34:57 $ 
 
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -26,6 +26,7 @@ use Language;
 use Log;
 use List;
 use Version;
+use Message;
 
 use Digest::MD5;
 use Fcntl;
@@ -70,8 +71,9 @@ my $sender = '';
 my $time_command;
 my $msg_file;
 
-#my $email_regexp = '(\S+|\".*\")(@\S+)';
-my $email_regexp = '\S+|\".*\"@\S+';
+## Caution : if this regexp changes (more/less parenthesis), then regexp using it should 
+## also be changed
+my $email_regexp = '([\w\-\_\.\/\+\=]+|\".*\")\@[\w\-]+(\.[\w\-]+)+';
 
 ## Parse the command and call the adequate subroutine with
 ## the arguments to the command.
@@ -81,7 +83,7 @@ sub parse {
    my $i = shift;
    my $sign_mod = shift;
 
-   do_log('debug', 'Commands::parse(%s, %s, %s, %s)', $sender, $robot, $i,$sign_mod );
+   do_log('debug2', 'Commands::parse(%s, %s, %s, %s)', $sender, $robot, $i,$sign_mod );
 
    my @msgsup = @_; ## For special commands (such as expire) needing
                     ## a message within a command
@@ -179,7 +181,7 @@ sub help {
 		close IN;
 	    }
 	}
-	push @msg::report, sprintf Msg(6, 70, "\nPowered by Sympa %s : http://listes.cru.fr/sympa/\n")
+	push @msg::report, sprintf Msg(6, 70, "\nPowered by Sympa %s : http://www.sympa.org/\n")
 	    , $Version ;
 
     }elsif (-r "--ETCBINDIR--/templates/helpfile.tpl") {
@@ -218,7 +220,7 @@ sub lists {
     my $sympa = &Conf::get_robot_conf($robot, 'sympa');
     my $host = &Conf::get_robot_conf($robot, 'host');
 
-    do_log('debug2', 'Commands::lists for robot %s', $robot);
+    do_log('debug', 'Commands::lists for robot %s', $robot);
 
     my $data = {};
     my $lists = {};
@@ -239,51 +241,11 @@ sub lists {
 	}
     }
 
-    # sa et si lists.$lang.tpl ?? Ca coince !!! Cela ne marche pas si le template existe !!!
-    if ((-r "$Conf{'etc'}/$robot/templates/lists.tpl") || (-r "$Conf{'etc'}/templates/lists.tpl") || (-r "--ETCBINDIR--/templates/lists.tpl")) {
-	my $data = {};
-	$data->{'lists'} = $lists;
-	$data->{'subject'} = MIME::Words::encode_mimewords(sprintf Msg(6, 82, "Public lists"));
-
-	&List::send_global_file('lists', $sender, $robot, $data);
-
-    }elsif (-r  'lists') {
-	## Static lists
-	open IN, 'lists';
-	while (<IN>) {
-	    push @msg::report, $_;
-	}
-	close IN;
-
-    }elsif ((-r 'lists.header') or (-r 'lists.footer')) {
-	## Header + Dynamic + Footer
-        my ($l, $list, $name, $subject);
-
-        ## Header file
-        if ( open HEADER, 'lists.header' ) {
-           while (<HEADER>) {
-	      push @msg::report, $_;
-           }
-           close HEADER;
-        }
- 
-	foreach $l (sort keys %{$lists}) {
-	   $name = $l . '.' x (22 - length($l));
-	   $subject = $lists->{$l}{'subject'};
-	   push @msg::report, "$name : $subject\n";
-	}
-
-        ## Footer file
-        if ( open FOOTER, 'lists.footer' ) {
-           while (<FOOTER>) {
-               push @msg::report, $_;
-           }
-           close FOOTER;
-        }
-    }else {
-        &do_log('info', 'Missing lists.tpl');
-        return undef;
-    }
+    my $data = {};
+    $data->{'lists'} = $lists;
+    $data->{'subject'} = MIME::Words::encode_mimewords(sprintf Msg(6, 82, "Public lists"));
+    
+    &List::send_global_file('lists', $sender, $robot, $data);
 
     do_log('info', 'LISTS from %s accepted (%d seconds)', $sender, time-$time_command);
 
@@ -295,7 +257,7 @@ sub stats {
     my $listname = shift;
     my $robot=shift;
 
-    do_log('debug2', 'Commands::stats(%s)', $listname);
+    do_log('debug', 'Commands::stats(%s)', $listname);
 
     my $list = new List ($listname, $robot);
     unless ($list) {
@@ -351,7 +313,7 @@ sub getfile {
     my($which, $file) = split(/\s+/, shift);
     my $robot=shift;
 
-    do_log('debug2', 'Commands::getfile(%s, %s)', $which, $file);
+    do_log('debug', 'Commands::getfile(%s, %s)', $which, $file);
 
     my $list = new List ($which, $robot);
     unless ($list) {
@@ -396,7 +358,7 @@ sub last {
 
     my $sympa = &Conf::get_robot_conf($robot, 'sympa');
 
-    do_log('debug2', 'Commands::last(%s, %s)', $which);
+    do_log('debug', 'Commands::last(%s, %s)', $which);
 
     my $list = new List ($which,$robot);
     unless ($list)  {
@@ -442,7 +404,7 @@ sub index {
     my $robot = shift;
 
 
-    do_log('debug2', 'Commands::index(%s) robot (%s)',$which,$robot);
+    do_log('debug', 'Commands::index(%s) robot (%s)',$which,$robot);
 
     my $list = new List ($which, $robot);
     unless ($list) {
@@ -497,11 +459,11 @@ sub review {
     if ($sign_mod eq 'smime') {
 	$auth_method='smime';
     }elsif ($auth ne '') {
-	do_log ('debug2',"auth received from $sender : $auth");
+	do_log ('debug',"auth received from $sender : $auth");
 	if ($auth eq $list->compute_auth ('','review')) {
 	    $auth_method='md5';
 	}else{
-            do_log ('debug', 'auth should be %s',$list->compute_auth ('','review'));
+            do_log ('debug2', 'auth should be %s',$list->compute_auth ('','review'));
 	    push @msg::report, sprintf Msg(6, 15, $msg::wrong_authenticator);
 	    do_log('info', 'REVIEW %s from %s refused, auth failed', $listname,$sender);
 	    return 'wrong_auth';
@@ -519,7 +481,7 @@ sub review {
 	unless (defined $action);
 
     if ($action =~ /request_auth/i) {
-	do_log ('debug',"auth requested from $sender");
+	do_log ('debug2',"auth requested from $sender");
         $list->request_auth ($sender,'review',$robot);
 	do_log('info', 'REVIEW %s from %s, auth requested (%d seconds)', $listname, $sender,time-$time_command);
 	return 1;
@@ -573,7 +535,7 @@ sub verify {
     my $robot = shift;
 
     my $sign_mod = shift ;
-    do_log('debug2', 'Commands::verify(%s)', $sign_mod );
+    do_log('debug', 'Commands::verify(%s)', $sign_mod );
     
     my $user;
     
@@ -581,7 +543,7 @@ sub verify {
     
     if ($sign_mod eq 'smime') {
 	$auth_method='smime';
-	do_log('info', 'VERIFY succefull from %s', $sender,time-$time_command);
+	do_log('info', 'VERIFY successfull from %s', $sender,time-$time_command);
 	push @msg::report, sprintf Msg(6, 262, "Your message signature as been succesfully verified using smime.\n");
     }else{
 	do_log('info', 'VERIFY from %s : could not find correct s/mime signature', $sender,time-$time_command);
@@ -599,7 +561,7 @@ sub subscribe {
 
     my $sign_mod = shift ;
 
-    do_log('debug2', 'Commands::subscribe(%s,%s)', $what,$sign_mod);
+    do_log('debug', 'Commands::subscribe(%s,%s)', $what,$sign_mod);
 
     $what =~ /^(\S+)(\s+(.+))?\s*$/;
     my($which, $comment) = ($1, $3);
@@ -643,7 +605,6 @@ sub subscribe {
     my $action = &List::request_action('subscribe',$auth_method,$robot,
 				       {'listname' => $which, 
 					'sender' => $sender });
-#    my $action = &List::get_action ('subscribe',$which,$sender,$auth_method);
     
     return undef
 	unless (defined $action);
@@ -661,12 +622,14 @@ sub subscribe {
 	return 'not_allowed';
     }
     if ($action =~ /owner/i) {
-	push @msg::report, sprintf Msg(6, 25, $msg::subscription_forwarded)
-	    unless ($action =~ /quiet/i);
+	push @msg::report, sprintf Msg(6, 25, $msg::subscription_forwarded);
 	## Send a notice to the owners.
-        my $keyauth = $list->compute_auth($sender,'add');
-	$list->send_sub_to_owner($sender, $keyauth, &Conf::get_robot_conf($robot, 'sympa'), $comment);
-	$list->store_susbscription_request($sender, $comment);
+	$list->send_notify_to_owner({'who' => $sender,
+				     'keyauth' => $list->compute_auth($sender,'add'),
+				     'replyto' => &Conf::get_robot_conf($robot, 'sympa'),
+				     'gecos' => $comment,
+				     'type' => 'subrequest'});
+	$list->store_subscription_request($sender, $comment);
 	do_log('info', 'SUB %s from %s forwarded to the owners of the list (%d seconds)', $which, $sender,time-$time_command);   
 	return 1;
     }
@@ -679,20 +642,16 @@ sub subscribe {
     }
     if ($action =~ /do_it/i) {
 
-	my $is_sub = $list->is_user($sender);
+	my $user_entry = $list->get_subscriber($sender);
 	
-	unless (defined($is_sub)) {
-	    do_log('info','User lookup failed');
-	    return undef;
-	}
-	
-	if ($is_sub) {
-	    
+	if (defined $user_entry) {
+		
 	    ## Only updates the date
 	    ## Options remain the same
 	    my $user = {};
 	    $user->{'update_date'} = time;
-	    $user->{'gecos'} = $comment if $comment;
+		$user->{'gecos'} = $comment if $comment;
+	    $user->{'subscribed'} = 1;
 	    
 	    return undef
 		unless $list->update_user($sender, $user);
@@ -704,6 +663,7 @@ sub subscribe {
 	    $u->{'email'} = $sender;
 	    $u->{'gecos'} = $comment;
 	    $u->{'date'} = $u->{'update_date'} = time;
+	    $u->{'subscribed'} = 1 if ($list->{'admin'}{'user_data_source'} eq 'include2');
 
 	    return undef  unless $list->add_user($u);
 	}
@@ -728,7 +688,9 @@ sub subscribe {
 
 	## If requested send notification to owners
 	if ($action =~ /notify/i) {
-	    $list->send_notify_to_owner($sender, $comment, 'subscribe');
+	    $list->send_notify_to_owner({'who' => $sender, 
+					 'gecos' =>$comment, 
+					 'type' => 'subscribe'});
 	}
 	do_log('info', 'SUB %s from %s accepted (%d seconds, %d subscribers)', $which, $sender, time-$time_command, $list->get_total());
 	
@@ -746,7 +708,7 @@ sub info {
     my $robot = shift;
     my $sign_mod = shift ;
 
-    do_log('debug2', 'Commands::info(%s,%s)', $listname,$robot);
+    do_log('debug', 'Commands::info(%s,%s)', $listname,$robot);
 
     my $sympa = &Conf::get_robot_conf($robot, 'sympa');
 
@@ -762,11 +724,11 @@ sub info {
     if ($sign_mod eq 'smime') {
 	$auth_method='smime';
     }elsif ($auth ne '') {
-	do_log ('debug',"auth received from $sender : $auth");
+	do_log ('debug2',"auth received from $sender : $auth");
 	if ($auth eq $list->compute_auth ('','info')) {
 	    $auth_method='md5';
 	}else{
-            do_log ('debug', 'auth should be %s',$list->compute_auth ('','info'));
+            do_log ('debug2', 'auth should be %s',$list->compute_auth ('','info'));
 	    push @msg::report, sprintf Msg(6, 15, $msg::wrong_authenticator);
 	    do_log('info', 'INFO %s from %s refused, auth failed', $listname,$sender);
 	    return 'wrong_auth';
@@ -776,7 +738,6 @@ sub info {
 	$auth_method='smtp';
     }
 
-    #my $action = &List::get_action('info',$listname,$sender,$auth_method);
     my $action = &List::request_action('info',$auth_method,$robot,
 				       {'listname' => $listname, 
 					'sender' => $sender });
@@ -822,7 +783,7 @@ sub info {
 	return 1;
     }
     if ($action =~ /request_auth/) {
-	do_log ('debug',"auth requested from $sender");
+	do_log ('debug2',"auth requested from $sender");
         $list->request_auth ($sender,'info', $robot);
 	do_log('info', 'REVIEW %s from %s, auth requested (%d seconds)', $listname, $sender,time-$time_command);
 	return 1;
@@ -841,7 +802,7 @@ sub signoff {
     my $robot = shift;
 
     my $sign_mod = shift ;
-    do_log('debug2', 'Commands::signoff(%s,%s)', $which,$sign_mod);
+    do_log('debug', 'Commands::signoff(%s,%s)', $which,$sign_mod);
 
     my ($l,$list,$auth_method);
     my $host = &Conf::get_robot_conf($robot, 'host');
@@ -890,7 +851,6 @@ sub signoff {
 	$auth_method='smtp';
     }  
     
-    #my $action = &List::get_action('unsubscribe',$which,$sender,$email,$auth_method);
     my $action = &List::request_action('unsubscribe',$auth_method,$robot,
 				       {'listname' => $which, 
 					'email' => $email,
@@ -921,8 +881,9 @@ sub signoff {
 	push @msg::report, sprintf Msg(6, 25, $msg::subscription_forwarded)
 	    unless ($action =~ /quiet/i);
 	## Send a notice to the owners.
-	my $keyauth = $list->compute_auth($sender,'del');
-	$list->send_sig_to_owner($sender, $keyauth);
+	$list->send_notify_to_owner({'who' => $sender,
+				     'keyauth' => $list->compute_auth($sender,'del'),
+				     'type' => 'sigrequest'});
 	do_log('info', 'SIG %s from %s forwarded to the owners of the list (%d seconds)', $which, $sender,time-$time_command);   
 	return 1;
     }
@@ -930,23 +891,38 @@ sub signoff {
 	## Now check if we know this email on the list and
 	## remove it if found, otherwise just reject the
 	## command.
-	unless ($list->is_user($email)) {
+	my $user_entry = $list->get_subscriber($email);
+	unless ((defined $user_entry) && ($user_entry->{'subscribed'} == 1)) {
 	    push @msg::report, sprintf Msg(6, 30, "Email address %s has not been found on the list %s. You did perhaps\nsubscribe using a different address ?\n"), $email, $list->{'name'};
 	    do_log('info', 'SIG %s from %s refused, not on list', $which, $email);
 	    
 	    ## Tell the owner somebody tried to unsubscribe
 	    if ($action =~ /notify/i) {
-		$list->send_notify_to_owner($email, $comment, 'warn-signoff');
+		$list->send_notify_to_owner({'who' => $email, 
+					     'gecos' => $comment, 
+					     'type' => 'warn-signoff'});
 	    }
 	    return 'not_allowed';
 	}
 	
-	## Really delete and rewrite to disk.
-	$list->delete_user($email);
+	if ($user_entry->{'included'} == 1) {
+	    unless ($list->update_user($email, 
+				       {'subscribed' => 0,
+					'update_date' => time})) {
+		do_log('info', 'SIG %s from %s failed, database update failed', $which, $email);
+		return undef;
+	    }
+
+	}else {
+	    ## Really delete and rewrite to disk.
+	    $list->delete_user($email);
+	}
 	
 	## Notify the owner
 	if ($action =~ /notify/i) {
-	    $list->send_notify_to_owner($email, $comment, 'signoff');
+	    $list->send_notify_to_owner({'who' => $email, 
+					 'gecos' => $comment, 
+					 'type' => 'signoff'});
 	}
 	
 	$list->save();
@@ -975,10 +951,10 @@ sub add {
 
     my $sign_mod = shift ;
 
-    do_log('debug2', 'Commands::add(%s,%s)', $what,$sign_mod );
+    do_log('debug', 'Commands::add(%s,%s)', $what,$sign_mod );
 
     $what =~ /^(\S+)\s+($email_regexp)(\s+(.+))?\s*$/;
-    my($which, $email, $comment) = ($1, $2, $4);
+    my($which, $email, $comment) = ($1, $2, $6);
     my $auth_method ;
 
     ## Load the list if not already done, and reject the
@@ -1007,7 +983,6 @@ sub add {
 	$auth_method='smtp';
     }
     
-    #my $action = &List::get_action ('add',$which,$sender,$email,$auth_method);
     my $action = &List::request_action('add',$auth_method,$robot,
 				       {'listname' => $which, 
 					'email' => $email,
@@ -1039,6 +1014,7 @@ sub add {
 	    my $user = {};
 	    $user->{'update_date'} = time;
 	    $user->{'gecos'} = $comment if $comment;
+	    $user->{'subscribed'} = 1;
 
 	    return undef 
 		unless $list->update_user($email, $user);
@@ -1050,9 +1026,10 @@ sub add {
 	    $u->{'email'} = $email;
 	    $u->{'gecos'} = $comment;
 	    $u->{'date'} = $u->{'update_date'} = time;
+	    $u->{'subscribed'} = 1 if ($list->{'admin'}{'user_data_source'} eq 'include2');
 	    
 	    return undef unless $list->add_user($u);
-	    $list->delete_susbscription_request($email);
+	    $list->delete_subscription_request($email);
 	    push @msg::report, sprintf Msg(6, 37, "User %s has been added to the list %s.\n"), $email, $which;
 	}
     
@@ -1076,7 +1053,10 @@ sub add {
 
 	do_log('info', 'ADD %s %s from %s accepted (%d seconds, %d subscribers)', $which, $email, $sender, time-$time_command, $list->get_total() );
 	if ($action =~ /notify/i) {
-	    $list->send_notify_to_owner($email, $comment,'add',$sender);
+	    $list->send_notify_to_owner({'who' => $email, 
+					 'gecos' => $comment,
+					 'type' => 'add',
+					 'by' => $sender});
 	}
 	return 1;
     }
@@ -1089,7 +1069,7 @@ sub invite {
     my $what = shift;
     my $robot=shift;
     my $sign_mod = shift ;
-    do_log('debug2', 'Commands::invite(%s,%s)', $what,$sign_mod);
+    do_log('debug', 'Commands::invite(%s,%s)', $what,$sign_mod);
 
     my $sympa = &Conf::get_robot_conf($robot, 'sympa');
 
@@ -1207,7 +1187,7 @@ sub remind {
     my $robot = shift;
     my $sign_mod = shift ;
 
-    do_log('debug2', 'Commands::remind(%s,%s)', $which,$sign_mod);
+    do_log('debug', 'Commands::remind(%s,%s)', $which,$sign_mod);
 
     my $host = &Conf::get_robot_conf($robot, 'host');
     
@@ -1235,7 +1215,7 @@ sub remind {
     if ($sign_mod eq 'smime') {
 	$auth_method='smime';
     }elsif ($auth ne '') {
-	do_log ('debug',"auth received from $sender : $auth");
+	do_log ('debug2',"auth received from $sender : $auth");
 
 	my $should_be;
 	if ($listname eq '*') {
@@ -1247,7 +1227,7 @@ sub remind {
 	if ($auth eq $should_be) {
 	    $auth_method = 'md5';
 	}else{
-            do_log ('debug', 'auth should be %s', $should_be);
+            do_log ('debug2', 'auth should be %s', $should_be);
 	    push @msg::report, sprintf Msg(6, 15, $msg::wrong_authenticator);
 	    do_log('info', 'REMIND %s from %s refused, auth failed', $listname,$sender);
 	    return 'wrong_auth';
@@ -1259,7 +1239,6 @@ sub remind {
     my $action;
 
     if ($listname eq '*') {
-	#$action = &List::get_action ('global_remind',$sender,$auth_method);
 	$action = &List::request_action('global_remind',$auth_method,$robot,
 					   {'sender' => $sender });
 	
@@ -1269,7 +1248,6 @@ sub remind {
 
 	$host = $list->{'admin'}{'host'};
 
-	#$action = &List::get_action ('remind',$listname,$sender,$auth_method);
 	$action = &List::request_action('remind',$auth_method,$robot,
 					   {'listname' => $listname, 
 					    'sender' => $sender });
@@ -1278,7 +1256,7 @@ sub remind {
     return undef
 	unless (defined $action);
 
-    if ($action =~ /reject(\(\'?(\w+)\'?\))/i) {
+    if ($action =~ /reject(\(\'?(\w+)\'?\))?/i) {
 	my $tpl = $2;
 	do_log ('info',"Remind for list $listname from $sender refused");
 	if ($tpl) {
@@ -1286,9 +1264,9 @@ sub remind {
 	}else {
 	    push @msg::report, sprintf Msg(6, 80, "You are not allowed to perform command %s in list %s\n"),'remind',$listname;
 	}
-	return 0;
+	return 'not_allowed';
     }elsif ($action =~ /request_auth/i) {
-	do_log ('debug',"auth requested from $sender");
+	do_log ('debug2',"auth requested from $sender");
 	if ($listname eq '*') {
 	    &List::request_auth ($sender,'remind', $robot);
 	}else {
@@ -1354,12 +1332,12 @@ sub remind {
 			
 			$global_info{$email} = $user;
 
-			do_log('debug','remind * : %s subscriber of %s', $email,$listname);
+			do_log('debug2','remind * : %s subscriber of %s', $email,$listname);
 			$count++ ;
 		    } 
 		} while ($user = $list->get_next_user());
 	    }
-	    do_log('debug','Sending REMIND * to %d users', $count);
+	    do_log('debug2','Sending REMIND * to %d users', $count);
 
 	    foreach my $email (keys %global_subscription) {
 		my $user = &List::get_user_db($email);
@@ -1395,7 +1373,7 @@ sub del {
 
     my $sign_mod = shift ;
 
-    do_log('debug2', 'Commands::del(%s,%s)', $what,$sign_mod);
+    do_log('debug', 'Commands::del(%s,%s)', $what,$sign_mod);
 
     $what =~ /^(\S+)\s+($email_regexp)\s*/;
     my($which, $who) = ($1, $2);
@@ -1439,9 +1417,6 @@ sub del {
     return undef
 	unless (defined $action);
 
-
-#    my $action = &List::get_action ('del', $which, $sender, $who, $auth_method);
-
     if ($action =~ /reject(\(\'?(\w+)\'?\))?/i) {
 	my $tpl = $2;
 	if ($tpl) {
@@ -1463,19 +1438,30 @@ sub del {
     if ($action =~ /do_it/i) {
 	## Check if we know this email on the list and remove it. Otherwise
 	## just reject the message.
-	unless ($list->is_user($who)) {
+	my $user_entry = $list->get_subscriber($who);
+
+	unless ((defined $user_entry) && ($user_entry->{'subscribed'} == 1)) {
 	    push @msg::report, sprintf Msg(6, 33, "Email address %s has not been found on the list.\n"), $who;
 	    do_log('info', 'DEL %s %s from %s refused, not on list', $which, $who, $sender);
 	    return 'not_allowed';
 	}
 	
 	## Get gecos before deletion
-	my $u = $list->get_subscriber($who);
-	my $gecos = $u->{'gecos'};
-
-	## Really delete and rewrite to disk.
-	my $u = $list->delete_user($who);
+	my $gecos = $user_entry->{'gecos'};
 	
+	if ($user_entry->{'included'} == 1) {
+	    unless ($list->update_user($who, 
+				       {'subscribed' => 0,
+					'update_date' => time})) {
+		do_log('info', 'DEL %s %s from %s failed, database update failed', $which, $who, $sender);
+		return undef;
+	    }
+
+	}else {
+	    ## Really delete and rewrite to disk.
+	    my $u = $list->delete_user($who);
+	}
+
 	$list->save();
 	
 	## Send a notice to the removed user, unless the owner indicated
@@ -1491,7 +1477,10 @@ sub del {
 	push @msg::report, sprintf Msg(6, 38, $msg::user_removed_from_list), $who, $which;
 	do_log('info', 'DEL %s %s from %s accepted (%d seconds, %d subscribers)', $which, $who, $sender, time-$time_command, $list->get_total() );
 	if ($action =~ /notify/i) {
-	    $list->send_notify_to_owner($who, "", 'del',$sender);
+	    $list->send_notify_to_owner({'who' => $who, 
+					 'gecos' => "", 
+					 'type' => 'del',
+					 'by' => $sender});
 	}
 	return 1;
     }
@@ -1506,7 +1495,7 @@ sub set {
     my $what = shift;
     my $robot = shift;
 
-    do_log('debug2', 'Commands::set(%s)', $what);
+    do_log('debug', 'Commands::set(%s)', $what);
 
     $what =~ /^\s*(\S+)\s+(\S+)\s*$/; 
     my ($which, $mode) = ($1, $2);
@@ -1635,45 +1624,35 @@ sub distribute {
     my $file = "$modqueue\/$name\_$key";
     
     ## if the file as been accepted by WWSympa, it's name is different.
-    if (! -r $file) {
+    unless (-r $file) {
         $file= "$modqueue\/$name\_$key.distribute";
     }
 
     ## Open and parse the file
-    if (!open(IN, $file)) {
-	push @msg::report, sprintf Msg(6, 41, "Unable to find the message of the list %s locked by the key %s.\nWarning : this message could have ever been send by another editor"),$name,$key;
-	do_log('info', 'DISTRIBUTE %s %s from %s refused, auth failed', $which, $key, $sender);
-	return 'wrong_auth';
+    my $message = new Message($file);
+    unless (defined $message) {
+	do_log('err', 'Unable to create Message object %s', $file);
+	push @msg::report, sprintf Msg(6, 41, "Unable to find the message of the list %s locked by the key %s.\nWarning : this message could have ever been send by another editor"),$name,$key ;
+	return 'msg_not_found';
     }
 
-
-    my $msg;
-    my $parser = new MIME::Parser;
-    $parser->output_to_core(1);
-    unless ($msg = $parser->read(\*IN)) {
-	do_log('notice', 'Unable to parse message');
-	return undef;
-    }
-
-    close(IN);
-    
-    my $bytes = -s $file;
-
-    ## If the message is crypted decrypt before distribute with crypt attribute
-    $is_crypted = 'not_crypted';
-    if (($msg->head->get('Content-Type') =~ /application\/(x-)?pkcs7-mime/i) && ($Conf{'openssl'})) {
-	    $is_crypted = 'smime_crypted';
-	    $file = '_ALTERED_';
-	    do_log('debug2','xxxxxxxxxxxxxxxxxxxxxxxx is crypted');
-	    unless ($msg = &tools::smime_decrypt ($msg,$list->{'name'})) {
-		do_log('debug','unable to decrypt message');
-		## xxxxx traitement d'erreur ?
-		return undef;
-	    };
-
-	}
-
+    my $msg = $message->{'msg'};
     my $hdr= $msg->head;
+
+    ## encrypted message
+    if ($message->{'smime_crypted'}) {
+	$is_crypted = 'smime_crypted';
+	$file = '_ALTERED_';
+	($msg, $file) = ($message->{'decrypted_msg'}, $message->{'decrypted_msg_as_string'});
+	unless (defined($msg)) {
+	    do_log('debug','unable to decrypt message');
+	    ## xxxxx traitement d'erreur ?
+	    return undef;
+	};
+	$hdr = $msg->head;
+	do_log('debug2', "message successfully decrypted");
+    }
+
     $hdr->add('X-Validation-by', $sender);
 
     ## Distribute the message
@@ -1697,70 +1676,41 @@ sub distribute {
 sub confirm {
     my $what = shift;
     my $robot = shift;
-
-    do_log('debug2', 'Commands::confirm(%s)', $what);
+    do_log('debug', 'Commands::confirm(%s)', $what);
 
     $what =~ /^\s*(\S+)\s*$/;
     my $key = $1;
     my $start_time = time; # get the time at the beginning
-    my $trouve = 0;
 
-    my $authqueue = $Conf{'queueauth'};
+    my $file;
 
-    unless (opendir DIR, $authqueue ) {
-	do_log('info', 'WARNING unable to read %s directory', $authqueue);
+    unless (opendir DIR, $Conf{'queueauth'} ) {
+        do_log('info', 'WARNING unable to read %s directory', $Conf{'queueauth'});
     }
 
-    my @qfile =  sort grep (!/^\./,readdir(DIR));
-    closedir DIR ;
-    my ($i, $curlist, $authdelay);
-
-    $authdelay = $Conf{'clean_delay_queueauth'};
-
-    # secure auth message configuration
-    $authdelay = 1 if ($authdelay<=0);
 
     # delete old file from the auth directory
-    foreach $i (sort @qfile) {
-	if ((stat "$authqueue/$i")[9] < (time -  $authdelay*86400) ){
-	    unlink ("$authqueue/$i") ;
-	    do_log('notice', 'Deleting unconfirmed message %s, too old', $i);
-	    next;
-	}
-	
-	if ($i=~/\_$key$/i){
-	    $trouve=1;
-	    $which = $`;
-	}
+    foreach (grep (!/^\./,readdir(DIR))) {
+        if (/\_$key$/i){
+	    $file= "$Conf{'queueauth'}\/$_";
+        }
+    }
+    closedir DIR ;
+    
+    unless ($file) {
+        push @msg::report, sprintf Msg(6, 68, "Unable to find the message auth locked by the key %s."),$key;
+        do_log('info', 'CONFIRM %s from %s refused, auth failed', $key,$sender);
+        return 'wrong_auth';
     }
 
-    my $file= "$authqueue\/$which\_$key";
-
-    ## Open and parse the file
-    if (!open(IN, $file) or $trouve==0) {
-	push @msg::report, sprintf Msg(6, 68, "Unable to find the message auth locked by the key %s."),$key;
-	do_log('info', 'CONFIRM %s from %s refused, auth failed', $key,$sender);
-	return 'wrong_auth';
-    }
-
-    my $msg;
-    my $parser = new MIME::Parser;
-    $parser->output_to_core(1);
-    unless ($msg = $parser->read(\*IN)) {
-	do_log('notice', 'Unable to parse message');
+    my $message = new Message ($file);
+    unless (defined $message) {
+	do_log('err', 'Unable to create Message object %s', $file);
 	return undef;
     }
 
-    close(IN);
- 
-    ## Load the list if not already done, and reject the
-    ## subscription if this list is unknown to us.
-    my $list = new List ($which, $robot);
-    unless ($list)  {
-	push @msg::report, sprintf Msg(6, 5, "List %s not found.\n"), $which;
-	do_log('info', 'CONFIRM %s from %s refused, unknown list for robot %s', $which,$sender,$robot);
-	return 'unknown_list';
-    }
+    my $msg = $message->{'msg'};
+    my $list = $message->{'list'};
 
     &Language::SetLang($list->{'admin'}{'lang'});
 
@@ -1769,7 +1719,6 @@ sub confirm {
     my $bytes = -s $file;
     my $hdr= $msg->head;
 
-    #my $action = &List::get_action ('send',$name,$sender,'md5',$hdr);
     my $action = &List::request_action('send','md5',$robot,
 				       {'listname' => $name, 
 					'sender' => $sender ,
@@ -1779,12 +1728,12 @@ sub confirm {
 	unless (defined $action);
 
     if ($action =~ /^editorkey/) {
-	my $key = $list->send_to_editor('md5', $msg, $file, 'not_crypted');
+	my $key = $list->send_to_editor('md5', $message);
 	do_log('info', 'Key %s for list %s from %s sent to editors', $key, $name, $sender);
 	$list->notify_sender($sender);
 	return 1;
     }elsif($action =~ /editor/){
-	my $key = $list->send_to_editor('smtp', $msg, $file, 'not_crypted');
+	my $key = $list->send_to_editor('smtp', $message);
 	do_log('info', 'Message for %s from %s sent to editors', $name, $sender);
 	$list->notify_sender($sender);
 	return 1;
@@ -1829,7 +1778,7 @@ sub reject {
     my $what = shift;
     my $robot = shift;
 
-    do_log('debug2', 'Commands::reject(%s)', $what);
+    do_log('debug', 'Commands::reject(%s)', $what);
 
     $what =~ /^(\S+)\s+(.+)\s*$/;
     my($which, $key) = ($1, $2);
@@ -1858,7 +1807,6 @@ sub reject {
 	return undef;
     }
 
-    # my $msg = new Mail::Internet [<IN>];
     close(IN);
     
     my $bytes = -s $file;
@@ -1870,7 +1818,6 @@ sub reject {
     
     ## Open the file
     if (!open(IN, $file)) {
-#    if (!-e $file) {
 	push @msg::report, sprintf Msg(6, 41, "Unable to find the message of the list %s locked by the key %s.\nWarning : this message could have ever been send by another editor"),$name,$key ;
 	do_log('info', 'REJECT %s %s from %s refused, auth failed', $which, $key, $sender);
 	return 'wrong_auth';
@@ -1887,7 +1834,6 @@ sub reject {
 	    return undef;
 	}
 
-#	my $message = new Mail::Internet [<IN>];
 	my @sender_hdr = Mail::Address->parse($message->head->get('From'));
         unless  ($#sender_hdr == -1) {
 	    my $rejected_sender = $sender_hdr[0]->address;
@@ -1912,7 +1858,7 @@ sub expire {
     my $what = shift;
     my $robot=shift;
 
-    do_log('debug2', 'Commands::expire(%s)', $what);
+    do_log('debug', 'Commands::expire(%s)', $what);
 
     my @msgexp = @_;
     my $name, $d1, $d2;
@@ -2134,7 +2080,7 @@ sub expireindex {
     my $robot = shift;
 
     $name =~ y/A-Z/a-z/;
-    do_log('debug2', 'Commands::expireindex(%s)', $name);
+    do_log('debug', 'Commands::expireindex(%s)', $name);
 
     ## Load the list if not already done, and reject the
     ## subscription if this list is unknown to us.
@@ -2230,7 +2176,7 @@ sub expiredel {
     my $name = shift;
     my $robot = shift;
     $name =~ y/A-Z/a-z/;
-    do_log('debug2', 'Commands::expiredel(%s)', $name);
+    do_log('debug', 'Commands::expiredel(%s)', $name);
 
     ## Load the list if not already done, and reject the
     ## subscription if this list is unknown to us.
@@ -2274,7 +2220,7 @@ sub expiredel {
 sub modindex {
     my $name = shift;
     my $robot = shift;
-    do_log('debug2', 'Commands::modindex(%s)', $name);
+    do_log('debug', 'Commands::modindex(%s)', $name);
     
     $name =~ y/A-Z/a-z/;
 
@@ -2376,8 +2322,9 @@ sub modindex {
 ## return information about the sender 
 sub which {
     my($listname, @which);
+    shift;
     my $robot = shift;
-    do_log('debug2', 'Commands::which(%s)', $listname);
+    do_log('debug', 'Commands::which(%s)', $listname);
     
     ## Subscriptions
     push @msg::report, sprintf  Msg(6, 67, "List of your current subscriptions : \n\n");    
