@@ -1,5 +1,5 @@
 # Conf.pm - This module does the sympa.conf and robot.conf parsing
-# RCS Identication ; $Revision: 1.45 $ ; $Date: 2002/01/23 08:45:05 $ 
+# RCS Identication ; $Revision: 1.64.2.3 $ ; $Date: 2003/06/26 07:54:03 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -25,6 +25,7 @@ package Conf;
 
 use Log;
 use Language;
+use wwslib;
 
 require Exporter;
 use Carp;
@@ -33,36 +34,40 @@ use Carp;
 @EXPORT = qw(%Conf);
 
 my @valid_options = qw(
-		       avg bounce_warn_rate bounce_halt_rate chk_cert_expiration_task expire_bounce
+		       avg bounce_warn_rate bounce_halt_rate chk_cert_expiration_task expire_bounce_task
 		       clean_delay_queue clean_delay_queueauth clean_delay_queuemod 
 		       cookie create_list crl_dir crl_update_task db_host db_env db_name db_options db_passwd db_type db_user 
 		       db_port db_additional_subscriber_fields db_additional_user_fields
-		       default_list_priority edit_list email etc
-		       global_remind home host domain lang listmaster log_socket_type 
-		       misaddressed_commands max_size maxsmtp msgcat nrcpt owner_priority pidfile spool queue 
+		       default_shared_quota default_archive_quota default_list_priority edit_list email etc
+		       global_remind home host domain lang listmaster log_socket_type log_level 
+		       misaddressed_commands misaddressed_commands_regexp max_size maxsmtp msgcat nrcpt owner_priority pidfile spool queue 
 		       queueauth queuetask queuebounce queuedigest queueexpire queuemod queuesubscribe queueoutgoing tmpdir
 		       loop_command_max loop_command_sampling_delay loop_command_decrease_factor
+		       purge_user_table_task
 		       remind_return_path request_priority rfc2369_header_fields sendmail sendmail_args sleep 
 		       sort sympa_priority syslog log_smtp umask welcome_return_path wwsympa_url
-                       openssl trusted_ca_options key_passwd ssl_cert_dir remove_headers
-		       antivirus_path antivirus_args anonymous_header_fields
+                       openssl capath cafile  key_passwd ssl_cert_dir remove_headers
+		       antivirus_path antivirus_args antivirus_notify anonymous_header_fields
 		       dark_color light_color text_color bg_color error_color selected_color shaded_color
 		       ldap_export_name ldap_export_host ldap_export_suffix ldap_export_password
 		       ldap_export_dnmanager ldap_export_connection_timeout
-		       list_check_smtp list_check_suffixes
+		       list_check_smtp list_check_suffixes  spam_protection web_archive_spam_protection
 );
+
+my %old_options = ('trusted_ca_options' => 'capath,cafile');
+
 my %valid_options = ();
 map { $valid_options{$_}++; } @valid_options;
 
 my %Default_Conf = 
     ('home'    => '--EXPL_DIR--',
-     'etc'     => '--DIR--/etc',
-     'trusted_ca_options' => '-CAfile --ETCBINDIR--/ca-bundle.crt',
+     'etc'     => '--ETCDIR--',
      'key_passwd' => '',
      'ssl_cert_dir' => '--EXPL_DIR--/X509-user-certs',
      'crl_dir' => '--EXPL_DIR--/crl',
      'umask'   => '027',
      'syslog'  => 'LOCAL1',
+     'log_level'  => 0,
      'nrcpt'   => 25,
      'avg'     => 10,
      'maxsmtp' => 20,
@@ -73,9 +78,9 @@ my %Default_Conf =
      'domain'  => undef,
      'email'   => 'sympa',
      'pidfile' => '--PIDDIR--/sympa.pid',
-     'msgcat'  => '--DIR--/nls',
+     'msgcat'  => '--NLSDIR--',
      'sort'    => 'fr,ca,be,ch,uk,edu,*,com',
-     'spool'   => '--DIR--/spool',
+     'spool'   => '--SPOOLDIR--',
      'queue'   => undef,
      'queuedigest'=> undef,
      'queuemod'   => undef,
@@ -111,10 +116,11 @@ my %Default_Conf =
      'owner_priority' => 9,
      'lang' => 'us',
      'misaddressed_commands' => 'reject',
+     'misaddressed_commands_regexp' => '(subscribe|unsubscribe|signoff)',
      'max_size' => 5242880,
      'edit_list' => 'owner',
      'create_list' => 'public_listmaster',
-     'global_remind' => '',
+     'global_remind' => 'listmaster',
      'wwsympa_url' => undef,
      'bounce_warn_rate' => '30',
      'bounce_halt_rate' => '50',
@@ -126,6 +132,7 @@ my %Default_Conf =
      'remove_headers' => 'Return-Receipt-To,Precedence,X-Sequence,Disposition-Notification-To',
      'antivirus_path' => '',
      'antivirus_args' => '',
+     'antivirus_notify' => 'sender',
      'anonymous_header_fields' => 'Sender,X-Sender,Received,Message-id,From,X-Envelope-To,Resent-From,Reply-To,Organization,Disposition-Notification-To,X-Envelope-From,X-X-Sender',
      'dark_color' => '#330099',
      'light_color' => '#ccccff',
@@ -143,10 +150,18 @@ my %Default_Conf =
      'ldap_export_dnmanager' => '',
      'ldap_export_connection_timeout' => '',
      'list_check_smtp' => '',
-     'list_check_suffixes' => 'request,owner,unsubscribe',
-     'expire_bounce' => 'daily'
+     'list_check_suffixes' => 'request,owner,editor,unsubscribe,subscribe',
+     'expire_bounce_task' => 'daily',
+     'purge_user_table_task' => 'monthly',
+     'default_archive_quota' => '',
+     'default_shared_quota' => '',
+     'capath' => '',
+     'cafile' => '',
+     'spam_protection' => 'javascript',
+     'web_archive_spam_protection' => 'cookie'
    );
    
+my $wwsconf;
 %Conf = ();
 
 ## Loads and parses the configuration file. Reports errors if any.
@@ -170,6 +185,8 @@ sub load {
 	    $value =~ s/\s*$//;
 	    ##  'tri' is a synonime for 'sort' (for compatibily with old versions)
 	    $keyword = 'sort' if ($keyword eq 'tri');
+	    ##  'key_password' is a synonime for 'key_passwd' (for compatibily with old versions)
+	    $keyword = 'key_passwd' if ($keyword eq 'key_password');
 	    ## Special case: `command`
 	    if ($value =~ /^\`(.*)\`$/) {
 		$value = qx/$1/;
@@ -177,7 +194,7 @@ sub load {
 	    }
 	    $o{$keyword} = [ $value, $line_num ];
 	}else {
-	    printf STDERR Msg(1, 3, "Malformed line %d: %s"), $config, $_;
+	    printf STDERR Msg(1, 3, "Malformed line %d in %s: %s"), $line_num, $config, $_;
 	    $config_err++;
 	}
     }
@@ -194,6 +211,9 @@ sub load {
     $o{'host'} = $o{'domain'} if (defined $o{'domain'}) ;
     $o{'domain'} = $o{'host'} if (defined $o{'host'}) ;
     
+    unless ( (defined $o{'cafile'}) || (defined $o{'capath'} )) {
+	$o{'cafile'} = '--ETCBINDIR--/ca-bundle.crt';
+    }   
     my $spool = $o{'spool'}[0] || $Default_Conf{'spool'};
 
     unless (defined $o{'queuedigest'}) {
@@ -224,7 +244,11 @@ sub load {
     ## Check if we have unknown values.
     foreach $i (sort keys %o) {
 	next if ($valid_options{$i});
-	printf STDERR  "Line %d, unknown field: %s in sympa.conf\n", $o{$i}[1], $i;
+	if ($old_options{$i}) {
+	    printf STDERR  "Line %d of sympa.conf, parameter %s is no more available, read documentation for new parameter(s) %s\n", $o{$i}[1], $i, $old_options{$i};
+	}else {
+	    printf STDERR  "Line %d, unknown field: %s in sympa.conf\n", $o{$i}[1], $i;
+	}
 	$config_err++;
     }
     ## Do we have all required values ?
@@ -293,6 +317,12 @@ sub load {
 
     @{$Conf{'listmasters'}} = split(/,/, $Conf{'listmaster'});
 
+    ## Set Regexp for accepted list suffixes
+    if (defined ($Conf{'list_check_suffixes'})) {
+	$Conf{'list_check_regexp'} = $Conf{'list_check_suffixes'};
+	$Conf{'list_check_regexp'} =~ s/,/\|/g;
+    }
+
     $Conf{'sympa'} = "$Conf{'email'}\@$Conf{'host'}";
     $Conf{'request'} = "$Conf{'email'}-request\@$Conf{'host'}";
     
@@ -312,8 +342,11 @@ sub load_robots {
 				  host            => 1,
 				  wwsympa_url     => 1,
 				  'title'         => 1,
+				  lang            => 1,
 				  default_home    => 1,
+				  cookie_domain   => 1,
 				  log_smtp        => 1,
+				  log_level       => 1,
 				  create_list     => 1,
 				  dark_color      => 1,
 				  light_color     => 1,
@@ -323,7 +356,15 @@ sub load_robots {
 				  selected_color  => 1,
 				  shaded_color    => 1,
 				  list_check_smtp => 1,
-				  list_check_suffixes => 1 );
+				  list_check_suffixes => 1,
+				  spam_protection => 1,
+				  web_archive_spam_protection => 1
+				  );
+
+    ## Load wwsympa.conf
+    unless ($wwsconf = &wwslib::load_config('--WWSCONFIG--')) {
+	print STDERR "Unable to load config file --WWSCONFIG--\n";
+    }
 
     unless (opendir DIR,$Conf{'etc'} ) {
 	printf STDERR "Unable to open directory $Conf{'etc'} for virtual robots config\n" ;
@@ -342,7 +383,8 @@ sub load_robots {
 	    next if (/^\s*$/o || /^[\#\;]/o);
 	    if (/^\s*(\S+)\s+(.+)\s*$/io) {
 		my($keyword, $value) = ($1, $2);
-		
+		$value =~ s/\s*$//;
+
 		$keyword = lc($keyword);
 		$value = lc($value) unless ($keyword eq 'title');
 		if ($valid_robot_key_words{$keyword}) {
@@ -361,18 +403,19 @@ sub load_robots {
 	## Default for 'host' is the domain
 	$robot_conf->{$robot}{'host'} ||= $robot;
 
+	$robot_conf->{$robot}{'title'} ||= $wwsconf->{'title'};
+	$robot_conf->{$robot}{'default_home'} ||= $wwsconf->{'default_home'};
+
+	$robot_conf->{$robot}{'lang'} ||= $Conf{'lang'};
 	$robot_conf->{$robot}{'email'} ||= $Conf{'email'};
-
 	$robot_conf->{$robot}{'log_smtp'} ||= $Conf{'log_smtp'};
-
+	$robot_conf->{$robot}{'log_level'} ||= $Conf{'log_level'};
 	$robot_conf->{$robot}{'wwsympa_url'} ||= 'http://'.$robot_conf->{$robot}{'http_host'}.'/wws';
-
 	$robot_conf->{$robot}{'sympa'} = $robot_conf->{$robot}{'email'}.'@'.$robot_conf->{$robot}{'host'};
-
 	$robot_conf->{$robot}{'request'} = $robot_conf->{$robot}{'email'}.'-request@'.$robot_conf->{$robot}{'host'};
-
+	$robot_conf->{$robot}{'cookie_domain'} ||= 'localhost';
 	$Conf{'robot_by_http_host'}{$robot_conf->{$robot}{'http_host'}} = $robot ;
-
+	
 
 	close (ROBOT_CONF);
     }
@@ -448,11 +491,11 @@ sub _load_auth {
     while (<IN>) {
  	
 	$line_num++;
-	next if (/^[\#\;]/o);	
+	next if (/^\s*[\#\;]/o);	
 	
       	if (/^\s*(\S+)\s*$/o) {$parag_name = $1;}
 	
-	if (/^\s*(\S+)\s+(\S+)\s*$/o){
+	if (/^\s*(\S+)\s+(.*\S)\s*$/o){
 	    
 	    my ($keyword,$value) = ($1,$2);
 	    $current_paragraph->{$keyword} = $value;
@@ -494,7 +537,7 @@ sub get_robot_conf {
     }
     
     ## default
-    return $Conf{$param};
+    return $Conf{$param} || $wwsconf->{$param};
 }
 
 ## Packages must return true.
