@@ -1,7 +1,7 @@
 # List.pm -  This module provides parsing functions for sympa template files.
 # These templates consists of text files including variables and directives
 # (IF, FOREACH, INCLUDE, PARSE,...)
-# RCS Identication ; $Revision: 1.5 $ ; $Date: 2002/01/17 12:53:05 $ 
+# RCS Identication ; $Revision: 1.16 $ ; $Date: 2002/10/03 13:53:40 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -21,16 +21,34 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-use FileHandle;
+package parser;
 
-my ($index, @t, $data, $internal);
+use FileHandle;
+use Log;
+
+my ($index, @t, $data, $internal, $previous_file, %option, $current_output);
 
 ## The main parsing sub
+## Parameters are   
+## data: a HASH ref containing the data   
+## template : a filename or a ARRAY ref that contains the template   
+## output : a Filedescriptor or a SCALAR ref for the output
 sub parse_tpl {
     my ($template, $output);
-    ($data, $template, $output) = @_;
+    ($data, $template, $output, $recurse) = @_;
 
-    print STDERR "\tsub parse_tpl $template\n" if $opt_p;
+    &do_log('debug2','Parser [%d] parse_tpl(%s)', $index, $template);
+
+    ## Reset loop cache unless recursive use
+    unless ($recurse == 1) {
+	$previous_file = undef;
+    }
+
+    ## Prevent loops
+    if ($previous_file eq $template) {
+	&do_log('err','Parser [%d] stopping loop with file %s', $index, $template);
+	return -1;
+    }
 
     my ($hash, $foreach);
     
@@ -40,8 +58,14 @@ sub parse_tpl {
     my @old_mode = ($*, $/);
     ($*, $/) = (0, "\n");
 
-    my $old_desc = select;
-    select $output;
+    my $old_desc;
+    if (ref($output) eq 'ARRAY') {           
+	$current_output = $output;       
+    }else {           
+	$current_output = $output;       
+	$old_desc = select;      
+	select $output;       
+    }
      
     ## Parses the HTML template
     ## Possible syntax of templates are 
@@ -51,17 +75,30 @@ sub parse_tpl {
     ## [FOREACH item IN list]...[item->NAME]...[END]
     ## [INCLUDE file]
     ## [PARSE file]
+    ## [STOPPARSE]...[STARTPARSE]
+    ## [SET var=value]
+    ## [SETOPTION opt]...[UNSETOPTION opt]
 
-    my $fh = new FileHandle $template;
+    my $fh;
 
-    $index = -1;
-    @t = <$fh>;
-    close $fh;
+    ## An array can be used as a template (instead of a filename)
+    if (ref($template) eq 'ARRAY') {           
+	@t = @$template;           
+	$index = -1;       
+    }else {
+	$fh = new FileHandle $template;
+	
+	$index = -1;
+	@t = <$fh>;
+	close $fh;
+    }
 
     &process(1);
 
-    select $old_desc;
-
+    unless (ref($output) eq 'ARRAY') {
+	select $old_desc;
+    }
+    
     ($*, $/) = @old_mode;
 
     ($index, $data) = ($old_index, $old_data);
@@ -70,6 +107,26 @@ sub parse_tpl {
 
 return 1;
 
+## Processes [SETOPTION xx] 
+sub do_setoption {
+
+    if (/\[\s*SETOPTION\s+(\w+)\s*\]/i) {
+	$option{$1} = 1;
+    }
+
+    return;
+}
+
+## Processes [UNSETOPTION xx] 
+sub do_unsetoption {
+
+    if (/\[\s*UNSETOPTION\s+(\w+)\s*\]/i) {
+	delete $option{$1};
+    }
+
+    return;
+}
+
 ## Processes [SET xx=yy] 
 sub do_setvar {
     my $echo = shift;
@@ -89,9 +146,21 @@ sub do_setvar {
 sub do_include {
     my $file = pop;
 
+    &do_log('debug2','Parser [%d] do_include(%s)', $index, $file);
+
+    if ($previous_file eq $file) {
+	&do_log('err','Parser [%d] stopping loop with file %s', $index, $file);
+	return -1;
+    }
+
     my $fh = new FileHandle $file;
 
-    print <$fh>;
+    if (ref($current_output) eq 'ARRAY') {
+	push @{$current_output}, sprintf <$fh>;
+    }else {
+	print <$fh>;
+    }
+
     close $fh;
 }
 
@@ -100,8 +169,7 @@ sub do_include {
 sub do_if {
     my ($echo) = @_;
 
-    print STDERR "[$index]\tsub do_if ($echo)\n" if $opt_p;
-    print STDERR "\t$t[$index]" if $opt_p;
+    &do_log('debug3','Parser [%d] do_if(%s)', $index, $t[$index]);
 
     $echo = -1 if ($echo == 1);
 
@@ -159,8 +227,7 @@ sub do_foreach {
     my ($echo) = @_;
     my ($i, $val, $var, $struct, $start);
 
-    print STDERR "[$index]\tsub do_foreach ($echo)\n" if $opt_p;
-    print STDERR "\t$t[$index]" if $opt_p;
+    &do_log('debug3','Parser [%d] do_foreach(%s)', $index, $t[$index]);
 
     if (/\[\s*FOREACH\s+(\w+)\s+IN\s+(\w+)(\->(\w+))?\s*\]/i) {
 	($var, $key, $key2) = ($1, $2, $4);
@@ -241,14 +308,19 @@ sub do_foreach {
 
 ## Processes [STOPPARSE]
 sub do_stopparse {
-    print STDERR "[$index]\tsub do_stopparse\n" if $opt_p;
-    print STDERR "\t$t[$index]" if $opt_p;
+
+    &do_log('debug3','Parser [%d] do_stopparse()', $index);
 
     $index++;
 
     while ($_ = $t[$index]) {
 	return if /\[\s*STARTPARSE\s*\]/i;
-	print;
+	
+	if (ref($current_output) eq 'ARRAY') {
+	    push @{$current_output}, sprintf $_;
+	}else {
+	    print;
+	}
 	$index++;
     }
     return;
@@ -262,11 +334,11 @@ sub process {
 
     $echo = 0 if ($echo == -1);
 
-    print STDERR "[$index]\tsub process ($echo)\n" if $opt_p;
-
     while ($_ = $t[++$index]) {
+	my $status;
+
 	if (/\[\s*IF.*\]/i) {
-	    &do_if($echo);
+	    $status = &do_if($echo);
 	}elsif (/\[\s*ENDIF\s*\]/i) {
 	    return;
 	}elsif (/\[\s*ELSE\s*\]/i) {
@@ -274,26 +346,37 @@ sub process {
 	}elsif (/\[\s*ELSIF\s*.*\]/i) {
 	    return;
 	}elsif (/\[\s*INCLUDE\s+(\w+)\s*\]/i) {
-	    &do_include($data->{$1}) if ($echo == 1);
+	    $status = &do_include($data->{$1}) if ($echo == 1);
 	}elsif (/\[\s*INCLUDE\s+\'(\S+)\'\s*\]/i) {
 	    &do_include($1) if ($echo == 1);
 	}elsif (/\[\s*PARSE\s+(\w+)\s*\]/i) {
-	    parse_tpl($data, $data->{$1}, select()) if ($echo == 1);
+	    $status = parse_tpl($data, $data->{$1}, select(), 1) if ($echo == 1);
 	}elsif (/\[\s*PARSE\s+(\w+)\->(\w+)\s*\]/i) {
-	    parse_tpl($data, $data->{$1}{$2}, select()) if (defined $data->{$1} && $echo == 1);
+	    $status = parse_tpl($data, $data->{$1}{$2}, select(), 1) if (defined $data->{$1} && $echo == 1);
 	}elsif (/\[\s*PARSE\s+\'(\S+)\'\s*\]/i) {
-	    parse_tpl($data, $1, select())  if ($echo == 1);
+	    $status = parse_tpl($data, $1, select(), 1)  if ($echo == 1);
 	}elsif (/\[\s*FOREACH\s+(\w+)\s+IN\s+(\w+(\->\w+)?)\s*\]/i) {
-	    &do_foreach($echo);
+	    $status = &do_foreach($echo);
 	}elsif (/\[\s*END\s*\]/i) {
 	    return;
 	}elsif (/\[\s*STOPPARSE\s*\]/i) {
-	    &do_stopparse();
+	    $status = &do_stopparse();
 	}elsif (/\[\s*SET\s+(\w+)\s*\=\s*(\w+\->\w+|\d+)\s*\]/i) {
-	    &do_setvar($echo);
+	    $status = &do_setvar($echo);
+	}elsif (/\[\s*SETOPTION\s+(\w+)\s*\]/i) {
+	    $status = &do_setoption();
+	}elsif (/\[\s*UNSETOPTION\s+(\w+)\s*\]/i) {
+	    $status = &do_unsetoption();
 	}elsif ($echo == 1) {
-	    &do_parse();
-	    print;
+	    $status = &do_parse();
+	    if (ref($current_output) eq 'ARRAY') {
+		push @{$current_output}, sprintf $_;
+	    }else {
+		print;
+	    }
+	}
+	if ($status == -1) {
+	    return -1;
 	}
     }
     return;
@@ -302,8 +385,7 @@ sub process {
 ## Instanciates variables
 sub do_parse {
 
-    print STDERR "[$index]\tsub do_parse\n" if $opt_p;
-    print STDERR "\t$t[$index]" if $opt_p;
+    &do_log('debug4','Parser [%d] parse(%s)', $index, $t[$index]);
 
     $_ = $t[$index];
     
@@ -314,20 +396,29 @@ sub do_parse {
 
 sub do_eval {
     my $var = shift;
+    
+    my $returned_value;
 
     if ($var =~ /^(\w+)\-\>(INDEX|NAME)$/) {
 	if (ref($internal->{$1}) eq 'HASH') {
-	    return $internal->{$1}{$2};
+	    $returned_value = $internal->{$1}{$2};
 	}
     }elsif ($var =~ /^(\w+)\-\>(\w+)$/) {
 	if (ref($data->{$1}) eq 'HASH') {
-	    return $data->{$1}{$2};
+	    $returned_value = $data->{$1}{$2};
 	}
     }elsif ($var =~ /^(\w+)$/) {
-	return $data->{$1};
+	$returned_value = $data->{$1};
     }else {
-	print STDERR "Unable to parse '$var'\n" if $opt_p;
+	&do_log('err','Parser [%d] unable to parse %s', $index, $var);
+	return '['.$var.']';
     }
 
-    return '';
+    ## If 'ignore_undef' option is set and result is undefined
+    ## Don't parse it
+    if ((! defined($returned_value)) && $option{'ignore_undef'}) {
+	return '['.$var.']';
+    }
+
+    return $returned_value;
 }
