@@ -1,5 +1,5 @@
 # smtp.pm - This module does the SMTP job, it does send messages
-# RCS Identication ; $Revision: 1.14 $ ; $Date: 2002/01/22 14:34:26 $ 
+# RCS Identication ; $Revision: 1.16.2.1 $ ; $Date: 2003/04/29 09:34:26 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -32,7 +32,7 @@ require 'tools.pl';
 use strict;
 
 ## RCS identification.
-#my $id = '@(#)$Id: smtp.pm,v 1.14 2002/01/22 14:34:26 salaun Exp $';
+#my $id = '@(#)$Id: smtp.pm,v 1.16.2.1 2003/04/29 09:34:26 salaun Exp $';
 
 my $opensmtp = 0;
 my $fh = 'fh0000000000';	## File handle for the stream.
@@ -61,13 +61,13 @@ sub reaper {
    while (($i = waitpid(-1, $block ? &POSIX::WNOHANG : 0)) > 0) {
       $block = 1;
       if (!defined($pid{$i})) {
-         print STDERR "Reaper waited $i, unknown process to me\n" if ($main::options{'debug'});
+         &do_log('debug2', "Reaper waited $i, unknown process to me");
          next;
       }
       $opensmtp--;
       delete($pid{$i});
    }
-   printf STDERR "Reaper unwaited pids : %s\nOpen = %s\n", join(' ', sort keys %pid), $opensmtp if ($main::options{'debug'});
+   &do_log('debug2', "Reaper unwaited pids : %s\nOpen = %s\n", join(' ', sort keys %pid), $opensmtp);
    return $i;
 }
 
@@ -103,9 +103,10 @@ sub smtpto {
    
    ## Check how many open smtp's we have, if too many wait for a few
    ## to terminate and then do our job.
-   print STDERR "Open = $opensmtp\n" if ($main::options{'debug'});
+
+   do_log('debug3',"Open = $opensmtp");
    while ($opensmtp > $Conf{'maxsmtp'}) {
-       print STDERR "Smtpto: too many open SMTP ($opensmtp), calling reaper\n" if ($main::options{'debug'});
+       do_log('debug3',"Smtpto: too many open SMTP ($opensmtp), calling reaper" );
        last if (&reaper(0) == -1); ## Blocking call to the reaper.
    }
 
@@ -135,7 +136,7 @@ sub smtpto {
        } else {
 	   $str .= join(' ', @$rcpt);
        }
-       do_log('debug', $str);
+       do_log('debug3', $str);
    }
    close(IN);
    $opensmtp++;
@@ -164,6 +165,9 @@ sub sendto {
 
     my $msg;
 
+    ## Encode subject before sending
+    $msg_header->replace('Subject', MIME::Words::encode_mimewords($msg_header->get('Subject')));
+
     if ($encrypt eq 'smime_crypted') {
 	my $email ;
 	if (ref($rcpt) eq 'SCALAR') {
