@@ -2,7 +2,7 @@
 
 # sympa.pl - This script is the main one ; it runs as a daemon and does
 # the messages/commands processing
-# RCS Identication ; $Revision: 1.68 $ ; $Date: 2002/01/23 08:45:05 $ 
+# RCS Identication ; $Revision: 1.104.2.5 $ ; $Date: 2003/04/29 09:50:44 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -40,14 +40,18 @@ use smtp;
 use MIME::QuotedPrint;
 use List;
 use Ldap;
+use Message;
 
 require 'tools.pl';
 require 'msg.pl';
 require 'parser.pl';
 
+
 # durty global variables
 my $is_signed = {}; 
 my $is_crypted ;
+# log_level is a global var, can be set by sympa.conf, robot.conf, list/config, --log_level or $PATHINFO  
+
 
 ## Internal tuning
 # delay between each read of the expirequeue
@@ -76,6 +80,7 @@ Options:
    -m, --mail          : log calls to sendmail
    --dump=list|ALL     : dumps subscribers 
    --lowercase         : lowercase email addresses in database
+   --log_level=LEVEL   : sets Sympa log level
 
    -h, --help          : print this help
    -v, --version       : print version number
@@ -88,12 +93,23 @@ encryption.
 
 ## Check --dump option
 my %options;
-&GetOptions(\%main::options, 'dump=s', 'debug|d', 'foreground', 'config|f=s', 
-	    'lang|l=s', 'mail|m', 'keepcopy|k=s', 'help', 'version', 'import=s', 'lowercase');
+&GetOptions(\%main::options, 'dump=s', 'debug|d', ,'log_level=s','foreground', 'config|f=s', 
+	    'lang|l=s', 'mail|m', 'keepcopy|k=s', 'help', 'version', 'import=s','make_alias_file','lowercase');
 
 
-$main::options{'debug2'} = 1 if ($main::options{'debug'});
+if ($main::options{'debug'}) {
+    $main::options{'log_level'} = 2 unless ($main::options{'log_level'});
+}
+# Some option force foreground mode
+$main::options{'foreground'} = 1 if ($main::options{'debug'} ||
+                                     $main::options{'version'} || 
+				     $main::options{'import'} ||
+				     $main::options{'help'} || 
+				     $main::options{'make_alias_file'} || 
+				     $main::options{'lowercase'} || 
+				     $main::options{'dump'});
 
+$log_level = $main::options{'log_level'} if ($main::options{'log_level'}); 
 
 my @parser_param = ($*, $/);
 my %loop_info;
@@ -113,7 +129,15 @@ unless (Conf::load($config_file)) {
 
 ## Open the syslog and say we're read out stuff.
 do_openlog($Conf{'syslog'}, $Conf{'log_socket_type'}, 'sympa');
-do_log('info', 'Configuration file read'); 
+
+# setting log_level using conf unless it is set by calling option
+if ($main::options{'log_level'}) {
+    do_log('info', "Configuration file read, log level set using options : $log_level"); 
+}else{
+    $log_level = $Conf{'log_level'};
+    do_log('info', "Configuration file read, default log level  $log_level"); 
+}
+
 
 ## Probe Db if defined
 if ($Conf{'db_name'} and $Conf{'db_type'}) {
@@ -127,6 +151,11 @@ if ($Conf{'db_name'} and $Conf{'db_type'}) {
 
 &tools::ciphersaber_installed();
 
+if (&tools::cookie_changed($Conf{'cookie'})) {
+     &fatal_err("sympa.conf/cookie parameter has changed. You may have severe inconsitencies into password storage. Restore previous cookie or write some tool to re-encrypt password in database and check spools contents (look at $Conf{'etc'}/cookies.history file)");
+    exit;
+}
+
 ## Set locale configuration
 $main::options{'lang'} =~ s/\.cat$//; ## Compatibility with version < 2.3.3
 $Language::default_lang = $main::options{'lang'} || $Conf{'lang'};
@@ -148,15 +177,13 @@ if ($signal ne 'hup' ) {
     ## works on many systems, although, it seems that Unix conceptors have
     ## decided that there won't be a single and easy way to detach a process
     ## from its controlling tty.
-    unless ($main::options{'debug'} || $main::options{'foreground'} ||
-	    $main::options{'dump'} || $main::options{'help'} ||
-	    $main::options{'version'} || $main::options{'import'} ||
-	    $main::options{'lowercase'} ) {
+    unless ($main::options{'foreground'}) {
 	if (open(TTY, "/dev/tty")) {
 	    ioctl(TTY, 0x20007471, 0);         # XXX s/b &TIOCNOTTY
 	    #       ioctl(TTY, &TIOCNOTTY, 0);
 	    close(TTY);
 	}
+	open(STDIN, ">> /dev/null");
 	open(STDERR, ">> /dev/null");
 	open(STDOUT, ">> /dev/null");
 	setpgrp(0, 0);
@@ -168,7 +195,7 @@ if ($signal ne 'hup' ) {
     }
     
     unless ($main::options{'dump'} || $main::options{'help'} ||
-	    $main::options{'version'} || $main::options{'import'} ||
+	    $main::options{'version'} || $main::options{'import'} || $main::options{'make_alias_file'} ||
 	    $main::options{'lowercase'} ) {
 	## Create and write the pidfile
 	&tools::write_pid($Conf{'pidfile'}, $$);
@@ -177,11 +204,11 @@ if ($signal ne 'hup' ) {
     do_openlog($Conf{'syslog'}, $Conf{'log_socket_type'}, 'sympa');
 
     # Set the UserID & GroupID for the process
-    $( = $) = (getpwnam('--GROUP--'))[2];
+    $( = $) = (getgrnam('--GROUP--'))[2];
     $< = $> = (getpwnam('--USER--'))[2];
 
     # Sets the UMASK
-    umask($Conf{'umask'});
+    umask(oct($Conf{'umask'}));
 
  ## Most initializations have now been done.
     do_log('notice', "Sympa $Version started");
@@ -213,6 +240,22 @@ if ($main::options{'dump'}) {
     exit 0;
 }elsif ($main::options{'help'}) {
     print $usage_string;
+    exit 0;
+}elsif ($main::options{'make_alias_file'}) {
+    my @listnames = &List::get_lists('*');
+    unless (open TMP, ">/tmp/sympa_aliases.$$") {
+	printf STDERR "Unable to create tmp/sympa_aliases.$$, exiting\n";
+	exit;
+    }
+    printf TMP "#\n#\tAliases for all Sympa lists (but not for robots)\n#\n";
+    close TMP;
+    foreach my $listname (@listnames) {
+	if (my $list = new List ($listname)) {
+
+	    system ("--SBINDIR--/alias_manager.pl add $list->{'name'} $list->{'domain'} /tmp/sympa_aliases.$$") ;
+	}	
+    }
+    printf ("Sympa aliases file is /tmp/sympa_aliases.$$ file made, you probably need to installed it in your SMTP engine\n");
     
     exit 0;
 }elsif ($main::options{'version'}) {
@@ -241,6 +284,8 @@ if ($main::options{'dump'}) {
 	%{$u} = %{$defaults};
 	$u->{'email'} = $email;
 	$u->{'gecos'} = $gecos;
+	$u->{'date'} = $u->{'update_date'} = time;
+	$u->{'subscribed'} = 1 if ($list->{'admin'}{'user_data_source'} eq 'include2');
 
 	unless ($list->add_user($u)) {
 	    printf STDERR "\nCould not add %s\n", $email;
@@ -281,7 +326,7 @@ if ($main::options{'keepcopy'}) {
 	&do_log('notice', 'Cannot keep a copy of incoming messages : %s is not a directory', $main::options{'keepcopy'});
 	delete $main::options{'keepcopy'};
     }elsif (! -w $main::options{'keepcopy'}) {
-	&do_log('notice', 'Cannot keep a copy of incoming messages : no write access to %s', $main::options{'keepcopy'});
+	&do_log('notice','Cannot keep a copy of incoming messages : no write access to %s', $main::options{'keepcopy'});
 	delete $main::options{'keepcopy'};
     }
 }
@@ -299,6 +344,13 @@ my @qfile;
 ## them, sleeps a while and continues the good job.
 while (!$signal) {
 
+    # setting log_level using conf unless it is set by calling option
+    unless ($main::options{'log_level'}) {
+	$log_level = $Conf{'log_level'};
+	# do_log('notice', "Reset default log level  $log_level"); 
+    }
+    
+
     &Language::SetLang($Language::default_lang);
 
     &List::init_list_cache();
@@ -324,6 +376,8 @@ while (!$signal) {
     if ($index_cleanqueue++ >= 100){
 	$index_cleanqueue=0;
 	&CleanSpool("$Conf{'queue'}/bad", $Conf{'clean_delay_queue'});
+	&CleanSpool($Conf{'queuemod'}, $Conf{'clean_delay_queuemod'});
+	&CleanSpool($Conf{'queueauth'}, $Conf{'clean_delay_queueauth'});
     }
 
     my $filename;
@@ -347,7 +401,7 @@ while (!$signal) {
 	if ($t_filename =~ /^BAD\-/i){
 	    if ((stat "$Conf{'queue'}/$t_filename")[9] < (time - $Conf{'clean_delay_queue'}*86400) ){
 		unlink ("$Conf{'queue'}/$t_filename") ;
-		do_log('notice', "Deleting bad message %s because too old", $t_filename);
+		do_log('notice',"Deleting bad message %s because too old", $t_filename);
 	    };
 	    next;
 	}
@@ -367,16 +421,17 @@ while (!$signal) {
 	}else{
 	    $t_robot = lc($Conf{'host'});
 	}
-	# do_log('debug', "listname %s    robot  %s", $t_listname,$t_robot);
 
-	if ($t_listname =~ /^(\S+)-(request|owner|editor|subscribe|unsubscribe)$/) {
+	my $list_check_regexp = &Conf::get_robot_conf($robot,'list_check_regexp');
+
+	if ($t_listname =~ /^(\S+)-($list_check_regexp)$/) {
 	    ($t_listname, $type) = ($1, $2);
 	}
 
 	# (sa) le terme "(\@$Conf{'host'})?" est inutile
-	unless ($t_listname =~ /^(sympa|listmaster|$Conf{'email'})(\@$Conf{'host'})?$/i) {
-	    $list = new List ($t_listname);
-	}
+	#unless ($t_listname =~ /^(sympa|listmaster|$Conf{'email'})(\@$Conf{'host'})?$/i) {
+	#    $list = new List ($t_listname);
+	#}
 	
 	if ($t_listname eq 'listmaster') {
 	    ## highest priority
@@ -388,6 +443,7 @@ while (!$signal) {
 	}elsif ($t_listname =~ /^(sympa|$Conf{'email'})(\@$Conf{'host'})?$/i) {	
 	    $priority = $Conf{'sympa_priority'};
 	}else {
+	    my $list =  new List ($t_listname);
 	    if ($list) {
 		$priority = $list->{'admin'}{'priority'};
 	    }else {
@@ -408,16 +464,20 @@ while (!$signal) {
 	next;
     }
 
-    do_log('debug', "Processing %s with priority %s", "$Conf{'queue'}/$filename", $highest_priority) if ($main::options{'debug'});
+    do_log('debug', "Processing %s with priority %s", "$Conf{'queue'}/$filename", $highest_priority) ;
     
     if ($main::options{'mail'} != 1) {
-		$main::options{'mail'} = $robot if ($Conf{'robots'}{$robot}{'log_smtp'});
+	$main::options{'mail'} = $robot if ($Conf{'robots'}{$robot}{'log_smtp'});
 	$main::options{'mail'} = $robot if ($Conf{'log_smtp'});
     }
+
+    ## Set NLS default lang for current message
+    $Language::default_lang = $main::options{'lang'} || &Conf::get_robot_conf($robot, 'lang');
+
     my $status = &DoFile("$Conf{'queue'}/$filename");
     
     if (defined($status)) {
-	do_log('debug', "Finished %s", "$Conf{'queue'}/$filename") if ($main::options{'debug'});
+	do_log('debug', "Finished %s", "$Conf{'queue'}/$filename") ;
 
 	if ($main::options{'keepcopy'}) {
 	    unless (rename "$Conf{'queue'}/$filename", $main::options{'keepcopy'}."/$filename") {
@@ -429,14 +489,22 @@ while (!$signal) {
 	}
     }else {
 	my $bad_dir = "$Conf{'queue'}/bad";
-	unless (-d $bad_dir) {
-	    do_log('err', "Missing directory '%s'", $bad_dir);
-	    rename("$Conf{'queue'}/$filename", "$Conf{'queue'}/BAD-$filename");
+
+	if (-d $bad_dir) {
+	    unless (rename("$Conf{'queue'}/$filename", "$bad_dir/$filename")){
+		do_log('err', "Exiting, unable to rename bad file %s to %s (check directory permission)", $filename, "$bad_dir/$filename");
+		exit;
+	    }
+	    do_log('notice', "Moving bad file %s to bad/", $filename);
+	}else{
+	    do_log('notice', "Missing directory '%s'", $bad_dir);
+	    unless (rename("$Conf{'queue'}/$filename", "$Conf{'queue'}/BAD-$filename")) {
+		do_log('err', "Exiting, unable to rename bad file %s to BAD-%s", $filename, $filename);
+		exit;		
+	    }
 	    do_log('notice', "Renaming bad file %s to BAD-%s", $filename, $filename);
 	}
-
-	rename("$Conf{'queue'}/$filename", "$bad_dir/$filename");
-	do_log('notice', "Moving bad file %s to bad/", $filename);
+	
     }
 
 } ## END of infinite loop
@@ -486,69 +554,63 @@ sub DoFile {
     
     my ($listname, $robot);
     my $status;
-    
-    ## Open and parse the file   
-    if (!open(IN, $file)) {
-	&do_log('info', 'Can\'t open %s: %m', $file);
-	return undef;
-    }
-    
-    my $parser = new MIME::Parser;
-    $parser->output_to_core(1);
 
-    my $msg;
-    unless ($msg = $parser->read(\*IN)) {
-	do_log('notice', 'Unable to parse message %s', $file);
+    my $message = new Message($file);
+    unless (defined $message) {
+	do_log('err', 'Unable to create Message object %s', $file);
 	return undef;
     }
+    
+    open TMP, ">/tmp/dump";
+    $message->dump(\*TMP);
+    close TMP;
+
+    my $msg = $message->{'msg'};
     my $hdr = $msg->head;
+    my $rcpt = $message->{'rcpt'};
     
     # message prepared by wwsympa and distributed by sympa
     if ( $hdr->get('X-Sympa-Checksum')) {
 	return (&DoSendMessage ($msg)) ;
     }
 
-    ## Ignoring messages without From: field
-    unless ($hdr->get('From')) {
-	do_log('notice', 'No From found in message, skipping.');
-	return undef;
-    }    
-
-    ## Search the X-Sympa-To header.
-    my $rcpt = $hdr->get('X-Sympa-To');
-    unless ($rcpt) {
-	do_log('notice', 'no X-Sympa-To found, ignoring message file %s', $file);
-	return undef;
-    }
-    chomp $rcpt;
-
     ## get listname & robot
     ($listname, $robot) = split(/\@/,$rcpt);
 
     $robot = lc($robot);
     $listname = lc($listname);
     $robot ||= $Conf{'host'};
+    
+    my $type;
+    my $list_check_regexp = &Conf::get_robot_conf($robot,'list_check_regexp');
+    if ($listname =~ /^(\S+)-($list_check_regexp)$/) {
+	($listname, $type) = ($1, $2);
+    }
+
+    # setting log_level using conf unless it is set by calling option
+    unless ($main::options{'log_level'}) {
+	$log_level = $Conf{'robots'}{$robot}{'log_level'};
+	do_log('debug', "Setting log level with $robot configuration (or sympa.conf) : $log_level"); 
+    }
+
+    ## Ignoring messages with no sender
+    my $sender = $message->{'sender'};
+    unless ($sender) {
+	do_log('err', 'No From found in message, skipping.');
+	return undef;
+    }
 
     ## Strip of the initial X-Sympa-To field
     $hdr->delete('X-Sympa-To');
-
-    my @sender_hdr = Mail::Address->parse($hdr->get('From'));
     
-    if ($#sender_hdr == -1) {
-	do_log('notice', 'No valid address in From: field, skipping');
-	return undef;
-    }
-
-    my $sender = $sender_hdr[0]->address;
-
     ## Loop prevention
     my $conf_email = &Conf::get_robot_conf($robot, 'email');
     my $conf_host = &Conf::get_robot_conf($robot, 'host');
-    if ($sender =~ /^(mailer-daemon|sympa|listserv|mailman|majordomo|smartlist|$conf_email)/mio) {
+    if ($sender =~ /^(mailer-daemon|sympa|listserv|mailman|majordomo|smartlist|$conf_email)(\@|$)/mio) {
 	do_log('notice','Ignoring message which would cause a loop, sent by %s', $sender);
 	return undef;
     }
-
+	
     ## Initialize command report
     undef @msg::report;  
     
@@ -556,9 +618,7 @@ sub DoFile {
     my $subject_field = &MIME::Words::decode_mimewords($hdr->get('Subject'));
     chomp $subject_field;
 #    $hdr->replace('Subject', $subject_field);
-    
-    my $bytes = -s $file;
-    
+        
     my ($list, $host, $name);   
     if ($listname =~ /^(sympa|listmaster|$conf_email)(\@$conf_host)?$/i) {
 	$host = $conf_host;
@@ -567,6 +627,11 @@ sub DoFile {
 	$list = new List ($listname);
 	$host = $list->{'admin'}{'host'};
 	$name = $list->{'name'};
+	# setting log_level using list config unless it is set by calling option
+	unless ($main::options{'log_level'}) {
+	    $log_level = $list->{'log_level'};
+	    do_log('debug', "Setting log level with list configuration : $log_level"); 
+	}
     }
     
     ## Loop prevention
@@ -587,62 +652,60 @@ sub DoFile {
     if ($hdr->get('Content-Identifier') =~ /Auto-replied/i) {
 	do_log('notice', "Ignoring message which would cause a loop (Content-Identifier: Auto-replied)");
 	return undef;
+    }elsif ($hdr->get('X400-Content-Identifier') =~ /Auto Reply to/i) {
+	do_log('notice', "Ignoring message which would cause a loop (X400-Content-Identifier: Auto Reply to)");
+	return undef;
     }
 
     ## encrypted message
-    $is_crypted = 'not_crypted';
-    if ($hdr->get('Content-Type') =~ /application\/(x-)?pkcs7-mime/i) {
-	do_log('debug2', "message is crypted");
-
-	if ($Conf{'openssl'}) {
-	    $is_crypted = 'smime_crypted';
-	    $file = '_ALTERED_';
-	    unless (($msg, $file) = &tools::smime_decrypt ($msg,$name)) {
-		do_log('debug','unable to decrypt message');
-		## xxxxx traitement d'erreur ?
-		return undef;
-	    };
-	    $hdr = $msg->head;
-	    do_log('debug2', "message successfully decrypted");
-	    # do_log('debug2', "xxxx dumped in /tmp/decrypted");
-	    # open (XXDUMP, ">/tmp/decrypted");
-	    # $msg->print(\*XXDUMP);
-	    # close(XXDUMP);
-	}
-
+    if ($message->{'smime_crypted'}) {
+	$is_crypted = 'smime_crypted';
+	$file = '_ALTERED_';
+	($msg, $file) = ($message->{'decrypted_msg'}, $message->{'decrypted_msg_as_string'});
+	unless (defined($msg)) {
+	    do_log('debug','unable to decrypt message');
+	    ## xxxxx traitement d'erreur ?
+	    return undef;
+	};
+	$hdr = $msg->head;
+	do_log('debug2', "message successfully decrypted");
+    }else {
+	$is_crypted = 'not_crypted';
     }
 
     ## S/MIME signed messages
-    undef $is_signed;
-    if ($Conf{'openssl'} && $hdr->get('Content-Type') =~ /multipart\/signed/i) {
-	$is_signed = &tools::smime_sign_check ($msg, $sender, $file);
-	do_log('debug2', "message is signed, signature is checked");
+    if ($message->{'smime_signed'}) {
+	$is_signed = {'subject' => $message->{'smime_subject'},
+		      'body' => 'smime'};
+    }else {
+	undef $is_signed;
     }
 
+
     if ($rcpt =~ /^listmaster(\@(\S+))?$/) {
 	$status = &DoForward('sympa', 'listmaster', $robot, $msg, $file, $sender);
 
 	## Mail adressed to the robot and mail 
 	## to <list>-subscribe or <list>-unsubscribe are commands
-    }elsif (($rcpt =~ /^(sympa|$conf_email)(\@\S+)?$/i) || ($rcpt =~ /^(\S+)-(subscribe|unsubscribe)(\@(\S+))?$/o)) {
+    }elsif (($rcpt =~ /^(sympa|$conf_email)(\@\S+)?$/i) || ($type =~ /^(subscribe|unsubscribe)$/o)) {
 	$status = &DoCommand($rcpt, $robot, $msg, $file);
 	
 	## forward mails to <list>-request <list>-owner etc
-    }elsif ($rcpt =~ /^(\S+)-(request|owner|editor)(\@(\S+))?$/o) {
-	my ($name, $function) = ($1, $2);
+#    }elsif ($rcpt =~ /^(\S+)-(request|owner|editor)(\@(\S+))?$/o) {
+    }elsif ($type =~ /^(request|owner|editor)$/o) {
+#	my ($name, $function) = ($1, $2);
 	
 	## Simulate Smartlist behaviour with command in subject
-        ## xxxxxxxxxxx  tendre le jeu de command reconnue sous cette forme ?
-        ## 
-	if (($function eq 'request') and ($subject_field =~ /^\s*(subscribe|unsubscribe)(\s*$name)?\s*$/i) ) {
+         ## 
+	if (($type eq 'request') and ($subject_field =~ /^\s*(subscribe|unsubscribe)(\s*$listname)?\s*$/i) ) {
 	    my $command = $1;
 	    
-	    $status = &DoCommand("$name-$command", $robot, $msg, $file);
+	    $status = &DoCommand("$listname-$command", $robot, $msg, $file);
 	}else {
-	    $status = &DoForward($name, $function, $robot, $msg, $file, $sender);
+	    $status = &DoForward($listname, $type, $robot, $msg, $file, $sender);
 	}       
     }else {
-	$status =  &DoMessage($rcpt, $msg, $robot,$bytes, $file, $is_crypted);
+	$status =  &DoMessage($rcpt, $message, $robot);
     }
     
 
@@ -708,7 +771,7 @@ sub DoFile {
 ## send a message as prepared by wwsympa
 sub DoSendMessage {
     my $msg = shift;
-    &do_log('debug2', 'DoSendMessage()');
+    &do_log('debug', 'DoSendMessage()');
 
     my $hdr = $msg->head;
     
@@ -740,7 +803,7 @@ sub DoSendMessage {
 ## Handles a message sent to [list]-editor, [list]-owner or [list]-request
 sub DoForward {
     my($name, $function, $robot, $msg, $file, $sender) = @_;
-    &do_log('debug2', 'DoForward(%s, %s, %s, %s)', $name, $function, $file, $sender);
+    &do_log('debug', 'DoForward(%s, %s, %s, %s)', $name, $function, $file, $sender);
 
     my $hdr = $msg->head;
     my $messageid = $hdr->get('Message-Id');
@@ -778,10 +841,8 @@ sub DoForward {
 	    unless (@rcpt);
 	
     }elsif ($function eq "request") {
-	foreach my $i (@{$admin->{'owner'}}) {
-	    next if ($i->{'reception'} eq 'nomail');
-	    push(@rcpt, $i->{'email'}) if ($i->{'email'});
-	}
+	@rcpt = $list->get_owners_email();
+
 	do_log('notice', 'Warning : no owner defined or all of them use nomail option in list %s', $name ) 
 	    unless (@rcpt);
 
@@ -792,10 +853,7 @@ sub DoForward {
 	}
 	unless (@rcpt) {
 	    do_log('notice', 'No editor defined in list %s (unless they use NOMAIL), use owners', $name ) ;
-	    foreach my $i (@{$admin->{'owner'}}) {
-		next if ($i->{'reception'} eq 'nomail');
-		push(@rcpt, $i->{'email'}) if ($i->{'email'});
-	    }
+	    @rcpt = $list->get_owners_email();
 	}
     }
     
@@ -808,19 +866,21 @@ sub DoForward {
     my $msg_copy = $msg->dup;
 
     if ($rc = &tools::virus_infected($msg_copy, $file)) {
-	if ($list) {
-	    $list->send_file('your_infected_msg', $sender, $robot, 
-			     {'virus_name' => $rc,
-			      'recipient' => $recepient.'@'.$host,
-			      'lang' => $list->{'admin'}{'lang'}});
+	if ($Conf{'antivirus_notify'} eq 'sender') {
+	    if ($list) {
+		$list->send_file('your_infected_msg', $sender, $robot, 
+				 {'virus_name' => $rc,
+				  'recipient' => $recepient.'@'.$host,
+				  'lang' => $list->{'admin'}{'lang'}});
+	    }
+	    else {
+		my %context;
+		$context{'virus_name'} = $rc ;
+		$context{'recipient'} = $recepient.'@'.$host;
+		$context{'lang'} = &Conf::get_robot_conf($robot, 'lang');
+		&List::send_global_file('your_infected_msg', $sender, $robot, \%context );
+	    }    
 	}
-	else {
-	    my %context;
-	    $context{'virus_name'} = $rc ;
-	    $context{'recipient'} = $recepient.'@'.$host;
-	    $context{'lang'} = $Conf{'lang'};
-	    &List::send_global_file('your_infected_msg', $sender, $robot, \%context );
-	}    
 	&do_log('notice', "Message for %s\@%s from %s ignored, virus %s found", $recepient, $host, $sender, $rc);
 
 	return undef;
@@ -838,8 +898,11 @@ sub DoForward {
 
 ## Handles a message sent to a list.
 sub DoMessage{
-    my($which, $msg, $robot, $bytes, $file, $encrypt ) = @_;
-    &do_log('debug2', 'DoMessage(%s, %s, %s, msg from %s, %s, %s,%s)', $which, $msg, $robot, $msg->head->get('From'), $bytes, $file, $encrypt);
+    my($which, $message, $robot) = @_;
+    my ($msg, $file, $bytes) = ($message->{'msg'}, $message->{'filename'}, $message->{'size'});
+    my $encrypt;
+    $encrypt = 'smime_crypted' if ($message->{'smime_crypted'});
+    &do_log('debug', 'DoMessage(%s, %s, %s, msg from %s, %s, %s,%s)', $which, $msg, $robot, $msg->head->get('From'), $bytes, $file, $encrypt);
     
     ## List and host.
     my($listname, $host) = split(/[@\s]+/, $which);
@@ -872,12 +935,14 @@ sub DoMessage{
 
     my $start_time = time;
     
+    &Language::SetLang($list->{'admin'}{'lang'});
+
     ## Now check if the sender is an authorized address.
 
     do_log('info', "Processing message for %s with priority %s, %s", $name,$list->{'admin'}{'priority'}, $messageid );
     
     my $conf_email = &Conf::get_robot_conf($robot, 'sympa');
-    if ($sender =~ /^(mailer-daemon|sympa|listserv|majordomo|smartlist|mailman|$conf_email)/mio) {
+    if ($sender =~ /^(mailer-daemon|sympa|listserv|majordomo|smartlist|mailman|$conf_email)(\@|$)/mio) {
 	do_log('notice', 'Ignoring message which would cause a loop');
 	return undef;
     }
@@ -929,10 +994,12 @@ sub DoMessage{
     my $rc;
    
     if ($rc= &tools::virus_infected($msg, $file)) {
-	printf "do message, virus= $rc \n";
-	$list->send_file('your_infected_msg', $sender, $robot, {'virus_name' => $rc,
-							'recipient' => $name.'@'.$host,
-							'lang' => $list->{'admin'}{'lang'}});
+	if ($Conf{'antivirus_notify'} eq 'sender') {
+	    #printf "do message, virus= $rc \n";
+	    $list->send_file('your_infected_msg', $sender, $robot, {'virus_name' => $rc,
+								    'recipient' => $name.'@'.$host,
+								    'lang' => $list->{'admin'}{'lang'}});
+	}
 	&do_log('notice', "Message for %s\@%s from %s ignored, virus %s found", $name, $host, $sender, $rc);
 	return undef;
     }
@@ -973,16 +1040,16 @@ sub DoMessage{
 	## the queue.
 	return 1;
     }elsif($action =~ /^request_auth/){
-    	my $key = $list->send_auth($sender, $msg, $file);
+    	my $key = $list->send_auth($message);
 	do_log('notice', 'Message for %s from %s kept for authentication with key %s', $name, $sender, $key);
 	return 1;
     }elsif($action =~ /^editorkey(\s?,\s?(quiet))?/){
-	my $key = $list->send_to_editor('md5',$msg,$file,$encrypt);
+	my $key = $list->send_to_editor('md5',$message);
 	do_log('info', 'Key %s for list %s from %s sent to editors, %s', $key, $name, $sender, $file, $encrypt);
 	$list->notify_sender($sender) unless ($2 eq 'quiet');
 	return 1;
     }elsif($action =~ /^editor(\s?,\s?(quiet))?/){
-	my $key = $list->send_to_editor('smtp', $msg, $file, $encrypt);
+	my $key = $list->send_to_editor('smtp', $message);
 	do_log('info', 'Message for %s from %s sent to editors', $name, $sender);
 	$list->notify_sender($sender) unless ($2 eq 'quiet');
 	return 1;
@@ -1031,6 +1098,13 @@ sub DoCommand {
     my @sender_hdr = Mail::Address->parse($from_field);
     my $sender = $sender_hdr[0]->address;
 
+    ## Detect loops
+    if ($msgid_table{$robot}{$messageid}) {
+	do_log('notice', 'Found known Message-ID, ignoring command which would cause a loop');
+	return undef;
+    }
+    $msgid_table{$robot}{$messageid}++;
+
     ## If X-Sympa-To = <listname>-<subscribe|unsubscribe> parse as a unique command
     if ($rcpt =~ /^(\S+)-(subscribe|unsubscribe)(\@(\S+))?$/o) {
 	do_log('debug',"processing message for $1-$2");
@@ -1080,7 +1154,7 @@ sub DoCommand {
 
     ## Process the body of the message
     ## unless subject contained commands or message has no body
-    unless (($success == 1) || (! defined $msg->bodyhandle)) { 
+    unless (defined($success) || (! defined $msg->bodyhandle)) { 
 #	foreach $i (@{$msg->body}) {
 	my @body = $msg->bodyhandle->as_lines();
 	foreach $i (@body) {
@@ -1155,7 +1229,7 @@ sub DoCommand {
 
 ## Read the queue and send old digests to the subscribers with the digest option.
 sub SendDigest{
-    &do_log('debug2', 'SendDigest()');
+    &do_log('debug', 'SendDigest()');
 
     if (!opendir(DIR, $Conf{'queuedigest'})) {
 	fatal_err(Msg(3, 1, "Can't open dir %s: %m"), $Conf{'queuedigest'}); ## No return.
@@ -1168,8 +1242,6 @@ sub SendDigest{
 
 	my $filename = $Conf{'queuedigest'}.'/'.$listname;
 
-	my @timedigest= (stat $filename)[9];
-
 	my $list = new List ($listname);
 	unless ($list) {
 	    &do_log('info', 'Unknown list, deleting digest file %s', $filename);
@@ -1194,7 +1266,7 @@ sub SendDigest{
 
 ## Read the EXPIRE queue and check if a process has ended
 sub ProcessExpire{
-    &do_log('debug2', 'ProcessExpire()');
+    &do_log('debug', 'ProcessExpire()');
 
     my $edir = $Conf{'queueexpire'};
     if (!opendir(DIR, $edir)) {
@@ -1301,7 +1373,7 @@ sub ProcessExpire{
 ## Clean old files from spool
 sub CleanSpool {
     my ($spool_dir, $clean_delay) = @_;
-    &do_log('debug2', 'CleanSpool(%s,%s)', $spool_dir, $clean_delay);
+    &do_log('debug', 'CleanSpool(%s,%s)', $spool_dir, $clean_delay);
 
     unless (opendir(DIR, $spool_dir)) {
 	do_log('err', "Unable to open '%s' spool : %s", $spool_dir, $!);
@@ -1315,8 +1387,23 @@ sub CleanSpool {
     foreach my $f (sort @qfile) {
 
 	if ((stat "$spool_dir/$f")[9] < (time - $clean_delay * 60 * 60 * 24)) {
-	    unlink ("$spool_dir/$f") ;
-	    do_log('notice', 'Deleting old file %s', "$spool_dir/$f");
+	    if (-f "$spool_dir/$f") {
+		unlink ("$spool_dir/$f") ;
+		do_log('notice', 'Deleting old file %s', "$spool_dir/$f");
+	    }elsif (-d "$spool_dir/$f") {
+		unless (opendir(DIR, "$spool_dir/$f")) {
+		    &do_log('err', 'Cannot open directory %s : %s', "$spool_dir/$f", $!);
+		    next;
+		}
+		my @files = sort grep (!/^\./,readdir(DIR));
+		foreach my $file (@files) {
+		    unlink ("$spool_dir/$f/$file");
+		}	
+		closedir DIR;
+		
+		rmdir ("$spool_dir/$f") ;
+		do_log('notice', 'Deleting old directory %s', "$spool_dir/$f");
+	    }
 	}
     }
 
@@ -1324,3 +1411,5 @@ sub CleanSpool {
 }
 
 1;
+
+
