@@ -1,7 +1,7 @@
 #! --PERL--
 
 # task_manager.pl - This script runs as a daemon and processes periodical Sympa tasks
-# RCS Identication ; $Revision: 1.22 $ ; $Date: 2002/01/23 12:52:29 $ 
+# RCS Identication ; $Revision: 1.46.2.1 $ ; $Date: 2003/01/22 09:35:56 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -33,7 +33,7 @@ use Conf;
 use Log;
 use Getopt::Long;
 use Time::Local;
-use MD5;
+use Digest::MD5;
 use smtp;
 use wwslib;
  
@@ -43,10 +43,22 @@ require 'tools.pl';
 my $opt_d;
 my $opt_F;
 my %options;
-&GetOptions(\%main::options, 'dump=s', 'debug|d', 'foreground', 'config|f=s', 
+
+&GetOptions(\%main::options, 'dump=s', 'debug|d', 'log_level=s', 'foreground', 'config|f=s', 
 	    'lang|l=s', 'mail|m', 'keepcopy|k=s', 'help', 'version', 'import=s', 'lowercase');
 
-$main::options{'debug2'} = 1 if ($main::options{'debug'});
+# $main::options{'debug2'} = 1 if ($main::options{'debug'});
+
+if ($main::options{'debug'}) {
+    $main::options{'log_level'} = 2 unless ($main::options{'log_level'});
+}
+# Some option force foreground mode
+$main::options{'foreground'} = 1 if ($main::options{'debug'} ||
+                                     $main::options{'version'} || 
+				     $main::options{'import'} ||
+				     $main::options{'help'} ||
+				     $main::options{'lowercase'} || 
+				     $main::options{'dump'});
 
 my $Version = '0.1';
 
@@ -68,14 +80,12 @@ my %regexp = ('email' => '(\S+|\".*\")(@\S+)',
 
 # Load WWSympa configuration
 unless ($wwsconf = &wwslib::load_config($wwsympa_conf)) {
-    &do_log ('err', 'error : unable to load config file');
-    exit;
+    &fatal_err('error : unable to load config file');
 }
 
 # Load sympa.conf
 unless (Conf::load($sympa_conf_file)) {
-    &do_log  ('err', "error : unable to load sympa configuration, file $sympa_conf_file has errors.");
-    exit(1);
+    &fatal_err("error : unable to load sympa configuration, file $sympa_conf_file has errors.");
 }
 
 ## Check databse connectivity
@@ -100,23 +110,32 @@ unless ($main::options{'debug'} || $main::options{'foreground'}) {
                                        
      setpgrp(0, 0);
      if ((my $child_pid = fork) != 0) {                                        
-         &do_log('debug', "Starting task_manager daemon, pid $_");	 
+         print STDOUT "Starting task_manager daemon, pid $_\n";	 
          exit(0);
      }     
  }
 
 &tools::write_pid($wwsconf->{'task_manager_pidfile'}, $$);
 
+$log_level = $main::options{'log_level'} || $Conf{'log_level'};
+
 $wwsconf->{'log_facility'}||= $Conf{'syslog'};
 do_openlog($wwsconf->{'log_facility'}, $Conf{'log_socket_type'}, 'task_manager');
 
-## Set the UserID & GroupID for the process
-$( = $) = (getpwnam('--USER--'))[2];
-$< = $> = (getpwnam('--GROUP--'))[2];
+# setting log_level using conf unless it is set by calling option
+if ($main::options{'log_level'}) {
+    do_log('info', "Configuration file read, log level set using options : $log_level"); 
+}else{
+    $log_level = $Conf{'log_level'};
+    do_log('info', "Configuration file read, default log level  $log_level"); 
+}
 
+## Set the UserID & GroupID for the process
+$( = $) = (getgrnam('--GROUP--'))[2];
+$< = $> = (getpwnam('--USER--'))[2];
 
 ## Sets the UMASK
-umask($Conf{'umask'});
+umask(oct($Conf{'umask'}));
 
 ## Change to list root
 unless (chdir($Conf{'home'})) {
@@ -134,8 +153,6 @@ my $end = 0;
 ###### VARIABLES DECLARATION ######
 
 my $spool_task = $Conf{'queuetask'};
-my $std_global_task_model_dir = "--ETCBINDIR--/global_task_models";
-my $user_global_task_model_dir = "$Conf{'etc'}/global_task_models";
 my $cert_dir = $Conf{'ssl_cert_dir'};
 my @tasks; # list of tasks in the spool
 
@@ -143,13 +160,15 @@ undef my $log; # won't execute send_msg and delete_subs commands if true, only l
 #$log = 1;
 
 ## list of list task models
-my @list_models = ('expire', 'remind');
+#my @list_models = ('expire', 'remind', 'sync_include');
+my @list_models = ('sync_include','remind');
 
 ## hash of the global task models
 my %global_models = (#'crl_update_task' => 'crl_update', 
 		     #'chk_cert_expiration_task' => 'chk_cert_expiration',
-		     'expire_bounce' => 'expire_bounce'
-		     #,'global_remind' => 'global_remind'
+		     'expire_bounce_task' => 'expire_bounce',
+		     'purge_user_table_task' => 'purge_user_table'
+		     #,'global_remind_task' => 'global_remind'
 		     );
 
 ## month hash used by epoch conversion routines
@@ -159,9 +178,9 @@ my %months = ('Jan', 0, 'Feb', 1, 'Mar', 2, 'Apr', 3, 'May', 4,  'Jun', 5,
 ###### DEFINITION OF AVAILABLE COMMANDS FOR TASKS ######
 
 my $date_arg_regexp1 = '\d+|execution_date';
-my $date_arg_regexp2 = '(\d\d\d\dy)(\d+m)?(\d+d)?(\d+h)?(\d+min)?'; 
-my $date_arg_regexp3 = '(\d+|execution_date)(\+|\-)(\d+y)?(\d+m)?(\d+w)?(\d+d)?(\d+h)?(\d+min)?';
-my $delay_regexp = '(\d+y)?(\d+m)?(\d+w)?(\d+d)?(\d+h)?(\d+min)?';
+my $date_arg_regexp2 = '(\d\d\d\dy)(\d+m)?(\d+d)?(\d+h)?(\d+min)?(\d+sec)?'; 
+my $date_arg_regexp3 = '(\d+|execution_date)(\+|\-)(\d+y)?(\d+m)?(\d+w)?(\d+d)?(\d+h)?(\d+min)?(\d+sec)?';
+my $delay_regexp = '(\d+y)?(\d+m)?(\d+w)?(\d+d)?(\d+h)?(\d+min)?(\d+sec)?';
 my $var_regexp ='@\w+'; 
 my $subarg_regexp = '(\w+)(|\((.*)\))'; # for argument with sub argument (ie arg(sub_arg))
                  
@@ -179,6 +198,8 @@ my %commands = ('next'                  => ['date', '\w*'],
 		                           #Number of days (delay)
 		'chk_cert_expiration'   => ['\w+', 'date'],
 		                           #template  date
+		'sync_include'          => [],
+		'purge_user_table'      => []
 		);
 
 # commands which use a variable. If you add such a command, the first parameter must be the variable
@@ -189,54 +210,29 @@ my %var_commands = ('delete_subs'      => ['var'],
 		    'rm_file'          => ['var'],
 		                          # variable
 		    );
-my @var_commands;
+
 foreach (keys %var_commands) {
     $commands{$_} = $var_commands{$_};
-    push (@var_commands, $_);
-}
-
+}                                     
+ 
 # commands which are used for assignments
 my %asgn_commands = ('select_subs'      => ['subarg'],
 		                            # condition
 		     'delete_subs'      => ['var'],
 		                            # variable
 		     );
-my @asgn_commands;
+
 foreach (keys %asgn_commands) {
     $commands{$_} = $asgn_commands{$_};
-    push (@asgn_commands, $_);
-}
-
-# list of all commands
-my @commands = keys %commands;
-
+}                                    
+     
 ###### INFINITE LOOP SCANING THE QUEUE (unless a sig TERM is received) ######
 while (!$end) {
     
     my $current_date = time; # current epoch date
     my $rep = &tools::adate ($current_date);
-    # &do_log ('notice', "****** $rep ******");
-
-    ## @tasks initialisation
-    unless (opendir(DIR, $spool_task)) {
-	&do_log ('err', "error : can't open dir %s: %m", $spool_task);
-    }
-    my @tasks = sort epoch_sort (grep !/^\.\.?$/, readdir DIR);
-
-    ## processing of tasks anterior to the current date
-    # &do_log ('debug2', 'processing of tasks anterior to the current date');
-    foreach my $task (@tasks) {
-	$task =~ /^(\d+)\.\w*\.\w+\.($regexp{'listname'}|_global)$/;
-	# &do_log ('debug2', "procesing %s/%s", $spool_task,$task);
-	last unless ($1 < $current_date);
-	if ($2 ne '_global') { # list task
-	    my $list = new List ($2);
-	    next unless ($list->{'admin'}{'status'} eq 'open');
-	}
-	execute ("$spool_task/$task");
-    }
-    # &do_log ('notice', 'done');
 
+    ## Create required tasks
     unless (opendir(DIR, $spool_task)) {
 	&do_log ('err', "error : can't open dir %s: %m", $spool_task);
     }
@@ -244,10 +240,11 @@ while (!$end) {
     @tasks = sort epoch_sort (grep !/^\.\.?$/, readdir DIR); # @tasks updating
     closedir DIR;
 
-    my @used_models; # models for which a task exists
+    my %used_models; # models for which a task exists
     foreach (@tasks) {
-	/.*\..*\.(.*)\..*/;
-	push (@used_models, $1) unless (in (\@used_models, $1));
+	if (my $task = &match_task($_)) {
+	    $used_models{$task->{'model'}} = 1;
+	}
     }
 
     ### creation of required tasks 
@@ -255,19 +252,18 @@ while (!$end) {
 			'execution_date' => 'execution_date');
 
     ## global tasks
-
     foreach my $key (keys %global_models) {
-	if (!in (\@used_models, $global_models{$key})) {
+	unless ($used_models{$global_models{$key}}) {
 	    if ($Conf{$key}) { 
 		my %data = %default_data; # hash of datas necessary to the creation of tasks
+		#printf "xxxxxxxxxxxxx appel 1\n";
 		create ($current_date, '', $global_models{$key}, $Conf{$key}, '_global', \%data);
-		push (@used_models, $1);
+		$used_models{$1} = 1;
 	    }
 	}
     }    
     
     ## list tasks
-
     foreach ( &List::get_lists() ) {
 	
 	my %data = %default_data;
@@ -279,25 +275,64 @@ while (!$end) {
 	foreach (@list_models) { $used_list_models{$_} = undef; }
 	
 	foreach $_ (@tasks) {
-	   /(.*)\.(.*)\.(.*)\.(.*)/;
-	   my $model = $3;
-	   my $object = $4;
-	   if ($object eq $list->{'name'}) { $used_list_models {$model} = 1; }
+	    if (my $task = &match_task($_)) {
+		my $model = $task->{'model'};
+		my $object = $task->{'list'};
+		if ($object eq $list->{'name'}) { $used_list_models{$model} = 1; }
+	    }
        }
         
 	foreach my $model (keys %used_list_models) {
 	    unless ($used_list_models{$model}) {
-		if ( $list->{'admin'}{$model.'_task'} ) {
-		    create ($current_date, '', $model, $list->{'admin'}{$model.'_task'}, 'list', \%data);
+		my $model_task_parameter = "$model".'_task';
+		
+		if ( $model eq 'sync_include') {
+		    next unless ($list->{'admin'}{'user_data_source'} eq 'include2');
+		    
+		    create ($current_date, 'INIT', $model, 'ttl', 'list', \%data);
+
+		}elsif ($list->{'admin'}{$model_task_parameter} ) {
+		    create ($current_date, '', $model, $list->{'admin'}{$model_task_parameter}{'name'}, 
+			    'list', \%data);
 		}
 	    }
 	}
     }
+
+    my $current_date = time; # current epoch date
+    my $rep = &tools::adate ($current_date);
+
+    ## Execute existing tasks
+    unless (opendir(DIR, $spool_task)) {
+	&do_log ('err', "error : can't open dir %s: %m", $spool_task);
+    }
+    my @tasks = sort epoch_sort (grep !/^\.\.?$/, readdir DIR);
+
+    ## processing of tasks anterior to the current date
+    &do_log ('debug3', 'processing of tasks anterior to the current date');
+    foreach (@tasks) {
+	if (my $task = &match_task($_)) {
+	    &do_log ('debug3', "procesing %s/%s", $spool_task,$_);
+	    last unless ($task->{'date'} < $current_date);
+	    if ($task->{'list'} ne '_global') { # list task
+		my $list = new List ($task->{'list'});
+		next unless ($list->{'admin'}{'status'} eq 'open');
+	    }
+	    execute ("$spool_task/$_");
+	}
+    }
+
     sleep 60;
     #$end = 1;
+
+    ## Free zombie sendmail processes
+    &smtp::reaper;
 }
 
-&do_log ('notice', 'task_manager exited normally due to signal');
+&do_log ('notice', 'task_manager exited normally due to signal'); 
+unless (unlink $wwsconf->{'task_manager_pidfile'}) { 
+    fatal_err("Could not delete %s, exiting", $wwsconf->{'task_manager_pidfile'}); 
+} 
 exit(0);
 
 ####### SUBROUTINES #######
@@ -305,12 +340,14 @@ exit(0);
 ## task creations
 sub create {
         
-    my $date          = $_[0];
-    my $label         = $_[1];
-    my $model         = $_[2];
-    my $model_choice  = $_[3];
-    my $object        = $_[4];
-    my $Rdata         = $_[5];
+    my $date          = shift;
+    my $label         = shift;
+    my $model         = shift;
+    my $model_choice  = shift;
+    my $object        = shift;
+    my $Rdata         = shift;
+
+    &do_log ('debug2', "create date : $date label : $label model $model : $model_choice object : $object Rdata :$Rdata");
 
     my $task_file;
     my $list_name;
@@ -328,11 +365,7 @@ sub create {
 
      # for global model
     if ($object eq '_global') {
-	if (open (MODEL, "$user_global_task_model_dir/$model_name")) {
-	    $model_file = "$user_global_task_model_dir/$model_name";
-	} elsif (open (MODEL, "$std_global_task_model_dir/$model_name")) {
-	    $model_file = "$std_global_task_model_dir/$model_name";
-	} else { 
+	unless ($model_file = &tools::get_filename('etc', "global_task_models/$model_name", $Conf{'host'})) {
 	    &do_log ('err', "error : unable to find $model_name, creation aborted");
 	    return undef;
 	}
@@ -342,14 +375,11 @@ sub create {
     if ($object  eq 'list') {
 	my $list = new List($list_name);
 
-	if (open (MODEL, "$list->{'dir'}/list_task_models/$model_name")) {
-	    $model_file = "$Conf{'home'}/$list_name/list_task_models/$model_name";
-	} elsif (open (MODEL, "$Conf{'etc'}/list_task_models/$model_name")) {
-	    $model_file = "$Conf{'etc'}/list_task_models/$model_name";
-	} elsif (open (MODEL, "--ETCBINDIR--/list_task_models/$model_name")) {
-	    $model_file = "--ETCBINDIR--/list_task_models/$model_name";
-	} else { 
-	    &do_log ('err', "error : unable to find $model_name, creation aborted");
+	$Rdata->{'list'}{'ttl'} = $list->{'admin'}{'ttl'};
+
+	unless ($model_file = &tools::get_filename('etc', "list_task_models/$model_name", 
+						   $list->{'domain'}, $list)) {
+	    &do_log ('err', "error : unable to find $model_name, for list $list_name creation aborted");
 	    return undef;
 	}
     }
@@ -359,7 +389,7 @@ sub create {
 
     ## creation
     open (TASK, ">$task_file");
-    parse_tpl ($Rdata, $model_file, TASK);
+    &parser::parse_tpl($Rdata, $model_file, \*TASK);
     close (TASK);
     
     # special checking for list whose user_data_source config parmater is include. The task won't be created if there is a delete_subs command
@@ -384,6 +414,10 @@ sub create {
 	} 
     } # end of special checking
 
+    if (!$ok) {
+	&do_log ('err', "$task_file is unappropriate for a list with include");
+    }
+    
     if  (!$ok or !check ($task_file)) {
 	&do_log ('err', "error : syntax error in $task_file, you should check $model_file");
 	unlink ($task_file) ? 
@@ -399,13 +433,15 @@ sub create {
 ## check the syntax of a task
 sub check {
 
-    my $task_file = $_[0]; # the task to check
+    my $task_file = shift; # the task to check
+
+    &do_log ('debug2', "check($task_file)" );
     my %result; # stores the result of the chk_line subroutine
     my $lnb = 0; # line number
-    my @used_labels; # list of labels used as parameter in commands
-    my @labels; # list of declared labels
-    my @used_vars; # list of vars used as parameter in commands
-    my @vars; # list of declared vars
+    my %used_labels; # list of labels used as parameter in commands
+    my %labels; # list of declared labels
+    my %used_vars; # list of vars used as parameter in commands
+    my %vars; # list of declared vars
 
     unless ( open (TASK, $task_file) ) {
 	&do_log ('err', "error : unable to read $task_file, checking is impossible");
@@ -418,6 +454,8 @@ sub check {
 	chomp;
 
 	$lnb++;
+
+	next if ( $_ =~ /^\s*\#/ ); 
 	unless (chk_line ($_, \%result)) {
 	    &do_log ('err', "error at line $lnb : $_");
 	    &do_log ('err', "$result{'error'}");
@@ -425,40 +463,41 @@ sub check {
 	}
 	
 	if ( $result{'nature'} eq 'assignment' ) {
-	    if (chk_cmd ($result{'command'}, $lnb, \@{$result{'Rarguments'}}, \@used_labels, \@used_vars)) {
-		push (@vars, $result{'var'});
-	    } else {return undef;}
+	    if (chk_cmd ($result{'command'}, $lnb, $result{'Rarguments'}, \%used_labels, \%used_vars)) {
+		$vars{$result{'var'}} = 1;
+	    } else {
+		return undef;}
 	}
-
+	
 	if ( $result{'nature'} eq 'command' ) {
-	    return undef unless (chk_cmd ($result{'command'}, $lnb, \@{$result{'Rarguments'}}, \@used_labels, \@used_vars));
+	    return undef unless (chk_cmd ($result{'command'}, $lnb, $result{'Rarguments'}, \%used_labels, \%used_vars));
 	} 
 			 
-	push (@labels, $result{'label'}) if ( $result{'nature'} eq 'label' );
+	$labels{$result{'label'}} = 1 if ( $result{'nature'} eq 'label' );
 	
     }
 
     # are all labels used ?
-    foreach my $label (@labels) {
-	&do_log ('notice', "warning : label $label exists but is not used") unless (in (\@used_labels, $label));
+    foreach my $label (keys %labels) {
+	&do_log ('notice', "warning : label $label exists but is not used") unless ($used_labels{$label});
     }
 
     # do all used labels exist ?
-    foreach my $label (@used_labels) {
-	unless (in (\@labels, $label)) {
+    foreach my $label (keys %used_labels) {
+	unless ($labels{$label}) {
 	    &do_log ('err', "error : label $label is used but does not exist");
 	    return undef;
 	}
     }
     
     # are all variables used ?
-    foreach my $var (@vars) {
-	&do_log ('notice', "warning : var $var exists but is not used") unless (in (\@used_vars, $var));
+    foreach my $var (keys %vars) {
+	&do_log ('notice', "warning : var $var exists but is not used") unless ($used_vars{$var});
     }
 
     # do all used variables exist ?
-    foreach my $var (@used_vars) {
-	unless (in (\@vars, $var)) {
+    foreach my $var (keys %used_vars) {
+	unless ($vars{$var}) {
 	    &do_log ('err', "error : var $var is used but does not exist");
 	    return undef;
 	}
@@ -472,6 +511,8 @@ sub chk_line {
 
     my $line = $_[0];
     my $Rhash = $_[1]; # will contain nature of line (label, command, error...)
+
+    &do_log('debug2', 'chk_line(%s, %s)', $line, $Rhash->{'nature'});
         
     $Rhash->{'nature'} = undef;
   
@@ -508,9 +549,9 @@ sub chk_line {
 	my @args = split (/,/, $2);
 	foreach (@args) { s/\s//g;}
 
-	unless (in (\@commands, $command)) { 
+	unless ($commands{$command}) { 
 	    $Rhash->{'nature'} = 'error';
-	    $Rhash->{'error'} = 'unknown command';
+	    $Rhash->{'error'} = "unknown command $command";
 	    return 0;
 	}
     
@@ -527,9 +568,9 @@ sub chk_line {
 
 	my %hash2;
 	chk_line ($2, \%hash2);
-	unless ( in (\@asgn_commands, $hash2{'command'}) ) { 
+	unless ( $asgn_commands{$hash2{'command'}} ) { 
 	    $Rhash->{'nature'} = 'error';
-	    $Rhash->{'error'} = 'non valid assignment';
+	    $Rhash->{'error'} = "non valid assignment $2";
 	    return 0;
 	}
 	$Rhash->{'nature'} = 'assignment';
@@ -552,50 +593,49 @@ sub chk_cmd {
     my $Rargs = $_[2]; # argument list
     my $Rused_labels = $_[3];
     my $Rused_vars = $_[4];
+
+    &do_log('debug2', 'chk_cmd(%s, %d, %s)', $cmd, $lnb, join(',',@{$Rargs}));
     
-    foreach my $command (@commands) { 
+    if (defined $commands{$cmd}) {
 	
-	if ($cmd eq $command) {
-
-	    my @expected_args = @{$commands{$command}};
-	    my @args = @{$Rargs};
-
-	    unless ($#expected_args == $#args) {
-		&do_log ('err', "error at line $lnb : wrong number of arguments for $command");
-		&do_log ('err', "args = @args ; expected_args = @expected_args");
+	my @expected_args = @{$commands{$cmd}};
+	my @args = @{$Rargs};
+	
+	unless ($#expected_args == $#args) {
+	    &do_log ('err', "error at line $lnb : wrong number of arguments for $cmd");
+	    &do_log ('err', "args = @args ; expected_args = @expected_args");
+	    return undef;
+	}
+	
+	foreach (@args) {
+	    
+	    undef my $error;
+	    my $regexp = $expected_args[0];
+	    shift (@expected_args);
+	    
+	    if ($regexp eq 'date') {
+		$error = 1 unless ( (/^$date_arg_regexp1$/i) or (/^$date_arg_regexp2$/i) or (/^$date_arg_regexp3$/i) );
+	    }
+	    elsif ($regexp eq 'delay') {
+		$error = 1 unless (/^$delay_regexp$/i);
+	    }
+	    elsif ($regexp eq 'var') {
+		$error = 1 unless (/^$var_regexp$/i);
+	    }
+	    elsif ($regexp eq 'subarg') {
+		$error = 1 unless (/^$subarg_regexp$/i);
+	    }
+	    else {
+		$error = 1 unless (/^$regexp$/i);
+	    }
+	    
+	    if ($error) {
+		&do_log ('err', "error at line $lnb : argument $_ is not valid");
 		return undef;
 	    }
-
-	    foreach (@args) {
-
-		undef my $error;
-		my $regexp = $expected_args[0];
-		shift (@expected_args);
-
-		if ($regexp eq 'date') {
-		    $error = 1 unless ( (/^$date_arg_regexp1$/i) or (/^$date_arg_regexp2$/i) or (/^$date_arg_regexp3$/i) );
-		}
-		elsif ($regexp eq 'delay') {
-		    $error = 1 unless (/^$delay_regexp$/i);
-		}
-		elsif ($regexp eq 'var') {
-		    $error = 1 unless (/^$var_regexp$/i);
-		}
-		elsif ($regexp eq 'subarg') {
-		    $error = 1 unless (/^$subarg_regexp$/i);
-		}
-		else {
-		    $error = 1 unless (/^$regexp$/i);
-		}
-
-		if ($error) {
-		    &do_log ('err', "error at line $lnb : argument $_ is not valid");
-		    return undef;
-		}
-
-		push (@{$Rused_labels}, $args[1]) if ($command eq 'next' && ($args[1]));   
-		push (@{$Rused_vars}, $args[0]) if (in (\@var_commands, $command));
-      	    }
+	    
+	    $Rused_labels->{$args[1]} if ($cmd eq 'next' && ($args[1]));   
+	    $Rused_vars->{$args[0]} = 1 if ($var_commands{$cmd});
 	}
     }
     return 1;
@@ -611,6 +651,8 @@ sub execute {
     my %vars; # list of task vars
     my $lnb = 0; # line number
 
+    &do_log('debug2', 'execute(%s, %d, %s)', $task_file, $lnb, join('/',  %vars));
+    
     unless ( open (TASK, $task_file) ) {
 	&do_log ('err', "error : can't read the task $task_file");
 	return undef;
@@ -631,6 +673,7 @@ sub execute {
     }
 
     # execution
+    my $status;
     while ( <TASK> ) {
   
 	chomp;
@@ -640,20 +683,31 @@ sub execute {
 	    &do_log ('err', "error : $result{'error'}");
 	    return undef;
 	}
-		
+	
 	# processing of the assignments
 	if ($result{'nature'} eq 'assignment') {
-	    $vars{$result{'var'}} = cmd_process ($result{'command'}, $result{'Rarguments'}, $task_file, \%vars, $lnb);
+	    $status = $vars{$result{'var'}} = &cmd_process ($result{'command'}, $result{'Rarguments'}, $task_file, \%vars, $lnb);
+	    last unless defined($status);
 	}
-					      
+	
 	# processing of the commands
 	if ($result{'nature'} eq 'command') {
-	    last unless cmd_process ($result{'command'}, $result{'Rarguments'}, $task_file, \%vars, $lnb);
+	    $status = &cmd_process ($result{'command'}, $result{'Rarguments'}, $task_file, \%vars, $lnb);
+	    last unless defined($status);
 	}
     } 
 
     close (TASK);
 
+    unless (defined $status) {
+	&do_log('err', 'Error while processing task, removing %s', $task_file);
+	unless (unlink($task_file)) {
+	    &do_log('err', 'Unable to remove task file %s : %s', $task_file, $!);
+	    return undef;
+	}
+	return undef;
+    }
+
     return 1;
 }
 
@@ -666,6 +720,8 @@ sub cmd_process {
     my $Rvars = $_[3]; # variable list of the task
     my $lnb = $_[4]; # line number
 
+    &do_log('debug2', 'cmd_process(%s, %s, %d)', $command, $task_file, $lnb);
+
      # building of %context
     my %context; # datas necessary to command processing
     $context{'task_file'} = $task_file; # long task file name
@@ -684,6 +740,8 @@ sub cmd_process {
     return exec_cmd ($Rarguments) if ($command eq 'exec');
     return update_crl ($Rarguments, \%context) if ($command eq 'update_crl');
     return expire_bounce ($Rarguments, \%context) if ($command eq 'expire_bounce');
+    return purge_user_table (\%context) if ($command eq 'purge_user_table');
+    return sync_include(\%context) if ($command eq 'sync_include');
 
      # commands which use a variable
     return send_msg ($Rarguments, $Rvars, \%context) if ($command eq 'send_msg');       
@@ -702,7 +760,7 @@ sub cmd_process {
  
  # remove files whose name is given in the key 'file' of the hash
 sub rm_file {
-        
+    
     my $Rarguments = $_[0];
     my $Rvars = $_[1];
     my $context = $_[2];
@@ -718,6 +776,8 @@ sub rm_file {
 	    return undef;
 	}
     }
+
+    return 1;
 }
 
 sub stop {
@@ -731,7 +791,7 @@ sub stop {
 	&do_log ('notice', "--> $task_file deleted")
 	    : error ($task_file, "error in stop command : unable to delete task file");
 
-    return undef;
+    return 0;
 }
 
 sub send_msg {
@@ -772,21 +832,69 @@ sub next_cmd {
     my @tab = @{$Rarguments};
     my $date = &tools::epoch_conv ($tab[0], $context->{'execution_date'}); # conversion of the date argument into epoch format
     my $label = $tab[1];
-    
+
     &do_log ('notice', "line $context->{'line_number'} of $context->{'task_name'} : next ($date, $label)");
 
-    $context->{'task_name'} =~ /\w*\.\w*\.(\w*)\.(($regexp{'listname'})|_global)/;
+    my @name = split /\./, $context->{'task_name'};
 
-    my $new_task = "$date.$label.$1.$2";
-    my $human_date = &tools::adate ($date);
-    my $new_task_file = "$spool_task/$new_task";
-    unless (rename ($context->{'task_file'}, $new_task_file)) {
-	error ("$context->{'task_file'}", "error in next command : unable to rename task file into $new_task");
+    ## Last item (listname) can contain '.' chars
+    $name[3] = join('.',@name[3..$#name]);
+    my $model = $name[2];
+
+    ## Determine type
+    my ($type, $model_choice);
+    my %data = ('creation_date'  => $context->{'execution_date'},
+		'execution_date' => 'execution_date');
+    if ($name[3] eq '_global') {
+	$type = '_global';
+	foreach my $key (keys %global_models) {
+	    if ($global_models{$key} eq $model) {
+		$model_choice = $Conf{$key};
+		last;
+	    }
+	}
+    }else {
+	$type = 'list';
+	my $list = new List($name[3]);
+	$data{'list'}{'name'} = $list->{'name'};
+	
+	if ( $model eq 'sync_include') {
+	    unless ($list->{'admin'}{'user_data_source'} eq 'include2') {
+		error ($context->{'task_file'}, "List $list->{'name'} no more require sync_include task");
+		return undef;
+	    }
+	    $data{'list'}{'ttl'} = $list->{'admin'}{'ttl'};
+	    $model_choice = 'ttl';
+	}else {
+	    unless (defined $list->{'admin'}{"$model\_task"}) {
+		error ($context->{'task_file'}, "List $list->{'name'} no more require $model task");
+		return undef;
+	    }
+
+	    $model_choice = $list->{'admin'}{"$model\_task"}{'name'};
+	}
+    }
+
+    unless (create ($date, $tab[1], $name[2], $model_choice, $type, \%data)) {
+	error ($context->{'task_file'}, "error in create command : creation subroutine failure");
 	return undef;
     }
-    &do_log ('notice', "--> new task $new_task ($human_date)");
+
+#    my $new_task = "$date.$label.$name[2].$name[3]";
+    my $human_date = &tools::adate ($date);
+#    my $new_task_file = "$spool_task/$new_task";
+#    unless (rename ($context->{'task_file'}, $new_task_file)) {
+#	error ("$context->{'task_file'}", "error in next command : unable to rename task file into $new_task");
+#	return undef;
+#    }
+    unless (unlink ($context->{'task_file'})) {
+	error ("$context->{'task_file'}", "error in next command : unable to remove task file $context->{'task_file'}");
+	return undef;
+    }
+
+    &do_log ('notice', "--> new task $model ($human_date)");
     
-    return undef;
+    return 0;
 }
 
 sub select_subs {
@@ -797,7 +905,7 @@ sub select_subs {
     my @tab = @{$Rarguments};
     my $condition = $tab[0];
  
-    &do_log ('debug', "line $context->{'line_number'} : select_subs ($condition)");
+    &do_log ('debug2', "line $context->{'line_number'} : select_subs ($condition)");
     $condition =~ /(\w+)\(([^\)]*)\)/;
     if ($2) { # conversion of the date argument into epoch format
 	my $date = &tools::epoch_conv ($2, $context->{'execution_date'});
@@ -808,17 +916,17 @@ sub select_subs {
     my %selection; # hash of subscribers who match the condition
     my $list = new List ($context->{'object_name'});
     
-    if ( $list->{'admin'}{'user_data_source'} =~ /database|file/) {
+    if ( $list->{'admin'}{'user_data_source'} =~ /database|file|include2/) {
         for ( my $user = $list->get_first_user(); $user; $user = $list->get_next_user() ) { 
             push (@users, $user);
 	}
     }
     
-    my $verify_context; # parameter of subroutine List::verify
-    $verify_context->{'sender'} = 'nobody' ;
-    $verify_context->{'email'} = $verify_context->{'sender'};
-    $verify_context->{'remote_host'} = 'unknown_host';
-    $verify_context->{'listname'} = $context->{'object_name'};
+    # parameter of subroutine List::verify
+    my $verify_context = {'sender' => 'nobody',
+			  'email' => 'nobody',
+			  'remote_host' => 'unknown_host',
+			  'listname' => $context->{'object_name'}};
     
     my $new_condition = $condition; # necessary to the older & newer condition rewriting
     # loop on the subscribers of $list_name
@@ -904,10 +1012,8 @@ sub create_cmd {
 	my $list = new List ($object);
 	$data{'list'}{'name'} = $list->{'name'};
     }
-    if ($type eq 'global') {
-	$type = '_global';
-    }
-
+    $type = '_global';
+    #printf "xxxxxxxxxxxxx appel 3\n";
     unless (create ($context->{'execution_date'}, '', $model, $model_choice, $type, \%data)) {
 	error ($context->{'task_file'}, "error in create command : creation subroutine failure");
 	return undef;
@@ -930,6 +1036,63 @@ sub exec_cmd {
     return 1;
 }
 
+sub purge_user_table {
+    my $Rarguments = $_[0];
+    my $context = $_[1];
+    do_log('debug2','purge_user_table()');
+
+    ## Load user_table entries
+    my @users = &List::get_all_user_db();
+
+    ## Load known subscribers/owners/editors
+    my %known_people;
+
+    ## Listmasters
+    foreach my $l (@{$Conf{'listmasters'}}) {
+	$known_people{$l} = 1;
+    }
+
+    foreach my $r (keys %{$Conf{'robots'}}) {
+	foreach my $l (&List::get_lists($r)){
+	    my $list = new List($l);
+	    next unless defined($list);
+
+	    ## Owners
+	    foreach my $o (@{$list->{'admin'}{'owner'}}) {
+		$known_people{$o->{'email'}} = 1;
+	    }
+
+	    ## Editors
+	    foreach my $e (@{$list->{'admin'}{'editor'}}) {
+		$known_people{$e->{'email'}} = 1;
+	    }
+	    
+	    ## Subscribers
+	    for (my $user = $list->get_first_user(); $user; $user = $list->get_next_user()) {
+		$known_people{$user->{'email'}} = 1;
+	    }
+	}
+    }    
+
+    ## Look for unused entries
+    my @purged_users;
+    foreach (@users) {
+	unless ($known_people{$_}) {
+	    &do_log('debug2','User to purge: %s', $_);
+	    push @purged_users, $_;
+	}
+    }
+    
+    unless ($#purged_users < 0) {
+	unless (&List::delete_user_db(@purged_users)) {
+	    &do_log('err', 'purge_user_table error: Failed to delete users');
+	    return undef;
+	}
+    }
+    
+    return $#purged_users + 1;
+}
+
 sub expire_bounce {
     # If a bounce is older then $list->get_latest_distribution_date()-$delai expire the bounce
     # Is this variable my be set in to task modele ?
@@ -940,12 +1103,12 @@ sub expire_bounce {
     my @tab = @{$Rarguments};
     my $delay = $tab[0];
 
-    do_log('debug2',"bounce expiration task  using $delay days as delay");
+    do_log('debug2','expire_bounce(%d)',$delay);
     foreach my $listname (&List::get_lists('*') ) {
 	my $list = new List ($listname);
 	# the reference date is the date until which we expire bounces in second
         # the latest_distribution_date is the date of last distribution #days from 01 01 1970
-	unless ( $list->{'admin'}{'user_data_source'} eq 'database' ) {
+	if ( ($list->{'admin'}{'user_data_source'} eq 'include' )||( $list->{'admin'}{'user_data_source'} eq 'file' )) {
 	    # do_log('notice','bounce expiration : skipping list %s because not using database',$listname);
 	    next;
 	}
@@ -1184,17 +1347,6 @@ sub epoch_sort {
     $date1 <=> $date2;
 }
 
-## return true if $element is in @tab
-sub in {
-    my $Rtab = $_[0];
-    my $element = $_[1];
-
-    foreach (@$Rtab) {
-	return 1 if ($element eq $_);
-    }
-    return undef;
-}
-
 ## change the label of a task file
 sub change_label {
     my $task_file = $_[0];
@@ -1222,3 +1374,42 @@ sub error {
     change_label ($task_file, 'ERROR') unless ($task_file eq '');
     &List::send_notify_to_listmaster ('error in task', $Conf{'domain'}, @param);
 }
+
+sub sync_include {
+    my $context = $_[0];
+
+    &do_log('debug2', 'sync_include(%s)', $context->{'object_name'});
+
+    my $list = new List($context->{'object_name'});
+    unless (defined $list) {
+	error ($context->{'task_file'}, "Unknown list $context->{'object_name'}");
+	return undef;
+    }
+
+    unless ( $list->{'admin'}{'user_data_source'} eq 'include2' ) {
+	&do_log('notice', 'sync_include() called for %s but user_data_source is %s', 
+		$list->{'name'}, $list->{'admin'}{'user_data_source'});                                                
+	return undef;                                                          
+    }
+ 
+
+    $list->sync_include();
+}
+
+## Check if the provided filename matches a task
+## Returns an array of its parts
+sub match_task {
+    my $filename = shift;
+
+    if ($filename =~ /^(\d+)\.(\w*)\.(\w+)\.($regexp{'listname'}|_global)$/) {
+	my $task = {'date' => $1,
+		    'label' => $2,
+		    'model' => $3,
+		    'list' => $4
+		};
+	return $task;
+    }
+    
+    return undef;
+}
+
