@@ -1,5 +1,5 @@
 # tools.pl - This module provides various tools for Sympa
-# RCS Identication ; $Revision: 1.63 $ ; $Date: 2002/01/23 08:43:34 $ 
+# RCS Identication ; $Revision: 1.90.2.10 $ ; $Date: 2003/07/17 07:47:48 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -30,7 +30,7 @@ use Log;
 use Time::Local;
 
 ## RCS identification.
-#my $id = '@(#)$Id: tools.pl,v 1.63 2002/01/23 08:43:34 salaun Exp $';
+#my $id = '@(#)$Id: tools.pl,v 1.90.2.10 2003/07/17 07:47:48 salaun Exp $';
 
 ## global var to store a CipherSaber object 
 my $cipher;
@@ -64,34 +64,11 @@ sub safefork {
    ## No return.
 }
 
-@avoid_hdr = (
-        'help',
-        'ind(ex)?',
-        'lists?',
-        '(please\s+)?(add|unsub?(scribe)?|remove|del|sub\s|sub?s?cribe|sign?o?f?f?)',
-        'rev(iew)?\s+\S+',
-        'stats\s+\S+',
-        'get\s+\S+\s+\S+',
-        'set\s+\S+\s+(no)?(mail|conceal|digest)',
-        'purge\s+\S+\s+\S+',
-        'exp(ire)?\s+\S+\s+\S+\s+\S+',
-        'exp(ire)?del\s+\S+',
-        'exp(ire)?ind(ex)?\s+\S+',
-        'ind(ex)?exp(ire)?\s+\S+',
-        'mod(eration)?ind(ex)?\s+\S+',
-        'ind(ex)?mod(eration)?\s+\S+',
-	'rev(iew)?\s+\S+',
-        'dis(tribute)?\s+\S+\s+\S+',
-        'rej(ect)?\s+\S+\s+\S+',
-        'confirm\s+\w+',
-	'rev(iew)?\s+\S+',
-);
-
 ## Check for commands in the body of the message. Returns true
 ## if there are some commands in it.
 sub checkcommand {
    my($msg, $sender, $robot) = @_;
-   do_log('debug2', 'tools::checkcommand(msg->head->get(subject): %s,%s)',$msg->head->get('Subject'), $sender);
+   do_log('debug3', 'tools::checkcommand(msg->head->get(subject): %s,%s)',$msg->head->get('Subject'), $sender);
 
    my($avoid, $i);
 
@@ -100,26 +77,22 @@ sub checkcommand {
    ## Check for commands in the subject.
    my $subject = $msg->head->get('Subject');
    if ($subject) {
-      foreach $avoid (@avoid_hdr) {
-         if ($subject =~ /^\s*(quiet)?($avoid)(\s+|$)/im) {
-            &rejectMessage($msg, $sender,$robot);
-            return 1;
-         }
-      }
+       if ($Conf{'misaddressed_commands_regexp'} && ($subject =~ /^$Conf{'misaddressed_commands_regexp'}$/im)) {
+	   &rejectMessage($msg, $sender,$robot);
+	   return 1;
+       }
    }
 
-   return 0 if ($#{$msg->body} >= 15);  ## More than 15 lines in the text.
+   return 0 if ($#{$msg->body} >= 5);  ## More than 5 lines in the text.
 
    foreach $i (@{$msg->body}) {
-      foreach $avoid (@avoid_hdr) {
-         if ($i =~ /^\s*(quiet)?($avoid)(\s+|$)/im) {  ## Suspicious line
-          #  &rejectMessage($msg, $sender);
-            return 1;
-         }
-      }
-      ## Control is only applied to first non-blank line
-      last unless $i =~ /^\s*$/;
-      
+       if ($Conf{'misaddressed_commands_regexp'} && ($i =~ /^$Conf{'misaddressed_commands_regexp'}$/im)) {
+	   &rejectMessage($msg, $sender, $robot);
+	   return 1;
+       }
+
+       ## Control is only applied to first non-blank line
+       last unless $i =~ /^\s*$/;
    }
    return 0;
 }
@@ -260,8 +233,9 @@ sub get_list_list_tpl {
     unless ($list_conf = &load_create_list_conf($robot)) {
 	return undef;
     }
-
-    foreach my $dir ('--ETCBINDIR--/create_list_templates', "$Conf{'etc'}/create_list_templates") {
+    
+    foreach my $dir ('--ETCBINDIR--/create_list_templates', "$Conf{'etc'}/create_list_templates",
+		     "$Conf{'etc'}/$robot/create_list_templates") {
 	if (opendir(DIR, $dir)) {
 	    foreach my $template ( sort grep (!/^\./,readdir(DIR))) {
 
@@ -290,8 +264,9 @@ sub smime_sign {
 
     do_log('debug2', 'tools::smime_sign (%s,%s)',$in_msg,$list);
 
-    my $cert = "$Conf{'home'}/$list/cert.pem";
-    my $key = "$Conf{'home'}/$list/private_key";
+    my $self = new List($list);
+    my $cert = $self->{'dir'}."/cert.pem";
+    my $key = $self->{'dir'}."/private_key";
     my $temporary_file = $Conf{'tmpdir'}."/".$list.".".$$ ;    
 
     my ($signed_msg,$pass_option );
@@ -320,8 +295,8 @@ sub smime_sign {
 	}
     }
 
-     &do_log('debug2', "$Conf{'openssl'} smime -sign -signer $cert $pass_option -inkey $key -in $temporary_file");
-     unless (open (NEWMSG,"$Conf{'openssl'} smime -sign -signer $cert $pass_option -inkey $key -in $temporary_file 2>&1 |")) {
+     &do_log('debug3', "$Conf{'openssl'} smime -sign -signer $cert $pass_option -inkey $key -in $temporary_file");
+     unless (open (NEWMSG,"$Conf{'openssl'} smime -sign -signer $cert $pass_option -inkey $key -in $temporary_file |")) {
     	&do_log('notice', 'Cannot sign message');
     }
 
@@ -344,7 +319,7 @@ sub smime_sign {
     }
     close NEWMSG ;
 
-    unlink ($temporary_file) unless ($main::options{'debug'} || $main::options{'debug2'}) ;
+    unlink ($temporary_file) unless ($main::options{'debug'}) ;
     
     ## foreach header defined in  the incomming message but undefined in the
     ## crypted message, add this header in the crypted form.
@@ -361,11 +336,10 @@ sub smime_sign {
 
 
 sub smime_sign_check {
+    my $message = shift;
 
-    my $msg = shift;
-    my $sender = shift;
-    my $file = shift;
-    $sender= lc($sender);
+    my $sender = $message->{'sender'};
+    my $file = $message->{'filename'};
 
     do_log('debug2', 'tools::smime_sign_check (message, %s, %s)', $sender, $file);
 
@@ -380,61 +354,90 @@ sub smime_sign_check {
 
 
 
-    my $temporary_file = "/tmp/smime-sender.".$$ ; 
-    do_log('debug2', "xxx $Conf{'openssl'} smime -verify  $Conf{'trusted_ca_options'} -signer  $temporary_file");
-    ## OpenSSL does not return error status ; should check first line of STDOUT
-    unless (open (MSGDUMP, "| $Conf{'openssl'} smime -verify  $Conf{'trusted_ca_options'} -signer $temporary_file > /dev/null")) {
+    my $temporary_file = "/tmp/smime-sender.".$$ ;
+    my $trusted_ca_options = '';
+    $trusted_ca_options = "-CAfile $Conf{'cafile'}" if ($Conf{'cafile'});
+    $trusted_ca_options = "-CApath $Conf{'capath'}" if ($Conf{'capath'});
+    do_log('debug3', "$Conf{'openssl'} smime -verify  $trusted_ca_options -signer  $temporary_file");
 
-	do_log('err', "unable to verify smime signature from $sender $verify");
+    unless (open (MSGDUMP, "| $Conf{'openssl'} smime -verify  $trusted_ca_options -signer $temporary_file > /dev/null")) {
+
+	do_log('err', "unabe to verify smime signature from $sender $verify");
 	return undef ;
     }
 
-#    unless (open MSG, $file) {
-#	do_log('err', 'Unable to open file %s: %s', $file, $!);
-#	return undef;
-#    }
+    if ($message->{'smime_crypted'}) {
+	print MSGDUMP ${$message->{'decrypted_msg_as_string'}};
+    }else {
+	unless (open MSG, $file) {
+	    do_log('err', 'Unable to open file %s: %s', $file, $!);
+	    return undef;
+	}
+	print MSGDUMP <MSG>;
+    }
 
-    $msg->print(\*MSGDUMP);
-    close MSGDUMP; close MSG;
+    close MSGDUMP;
+    my $status = $?/256 ;
+    close MSG;
+    if ($status == '1') {
+	do_log('err', "Openssl ERROR while parsing the command options : $Conf{'openssl'} smime -verify  $Conf{'trusted_ca_options'} -signer  $temporary_file");
+	return undef ;
+    }
+    if ($status == '2') {
+	do_log('err', "One of input file could not be read : $Conf{'openssl'} smime -verify  $Conf{'trusted_ca_options'} -signer  $temporary_file");
+	return undef ;
+    }
+    if ($status == '3') {
+	do_log('err', "Error creating the PKCS#7 file: $Conf{'openssl'} smime -verify  $Conf{'trusted_ca_options'} -signer  $temporary_file");
+	return undef ;
+    }
+    if ($status == '4') {
+	do_log('err', "Error while verifying the message signature. The message may have been corrupted");
+	return undef ;
+    }
+    if ($status == '5') {
+	do_log('err', "The message signature is checked but could not write out the signers certificates");
+    }    
     
     ## second step is the message signer match the sender
     ## a better analyse should be performed to extract the signer email. 
     my $signer = `cat $temporary_file | $Conf{'openssl'}  x509 -subject -noout`;
+    chomp $signer;
 
-
-    if ($signer =~ /email=$sender/i) {
-	do_log('debug', "S/MIME signed message, signature checked and sender match signer(%s)",$signer);
-        ## store the signer certificat
-	unless (-d $Conf{'ssl_cert_dir'}) {
-	    if ( mkdir ($Conf{'ssl_cert_dir'}, 0775)) {
-		do_log('info', "creating spool $Conf{'ssl_cert_dir'}");
-	    }else{
-		do_log('err', "Unable to create user certificat directory $Conf{'ssl_cert_dir'}");
-	    }
-	}
-	my $filename = "$Conf{'ssl_cert_dir'}/".&escape_chars($sender);
-
-	open (CERTIF,$temporary_file);
-	if (open (USERCERTIF, "> $filename")) {
-	    print USERCERTIF <CERTIF> ;
-	    close USERCERTIF ;
-	}else{
-	    &do_log('err','Unable to rename %s %s',$temporary_file,$filename);
-	}
-	close CERTIF;
-	unlink($temporary_file) unless ($main::options{'debug'} || $main::options{'debug2'}) ;	
-
-	$is_signed->{'body'} = 'smime';
-
-	# futur version should check if the subject was part of the SMIME signature.
-	$is_signed->{'subject'} = undef;
-	return $is_signed;
-    }else{
-	unlink($temporary_file) unless ($main::options{'debug'} || $main::options{'debug2'}) ;	
+    unless ($signer =~ /email=$sender/i) {
+	unlink($temporary_file) unless ($main::options{'debug'}) ;	
 	do_log('notice', "S/MIME signed message, sender($sender) do NOT match signer($signer)",$sender,$signer);
 	return undef;
     }
-    return undef ;    
+
+    do_log('debug', "S/MIME signed message, signature checked and sender match signer(%s)",$signer);
+    ## store the signer certificat
+    unless (-d $Conf{'ssl_cert_dir'}) {
+	if ( mkdir ($Conf{'ssl_cert_dir'}, 0775)) {
+	    do_log('info', "creating spool $Conf{'ssl_cert_dir'}");
+	}else{
+	    do_log('err', "Unable to create user certificat directory $Conf{'ssl_cert_dir'}");
+	}
+    }
+    my $filename = "$Conf{'ssl_cert_dir'}/".&escape_chars($sender);
+    
+    open (CERTIF,$temporary_file);
+    if (open (USERCERTIF, "> $filename")) {
+	print USERCERTIF <CERTIF> ;
+	close USERCERTIF ;
+    }else{
+	&do_log('err','Unable to rename %s %s',$temporary_file,$filename);
+    }
+    close CERTIF;
+    
+    
+    unlink($temporary_file) unless ($main::options{'debug'}) ;	
+    
+    $is_signed->{'body'} = 'smime';
+    
+    # futur version should check if the subject was part of the SMIME signature.
+    $is_signed->{'subject'} = $signer;
+    return $is_signed;
 }
 
 # input : msg object, return a new message object encrypted
@@ -448,9 +451,10 @@ sub smime_encrypt {
     my $cryptedmsg;
     my $encrypted_body;    
 
-    &do_log('debug2', 'tools::smime_encrypt(%s, %s, %s, %s',$msg_header, $msg_body, $email, $list);
+    &do_log('debug2', 'tools::smime_encrypt( %s, %s', $email, $list);
     if ($list eq 'list') {
-	$usercert = "$Conf{'home'}/$email/cert.pem";
+	my $self = new List($email);
+	$usercert = $self->{'dir'}."/cert.pem";
     }else{
 	$usercert = "$Conf{'ssl_cert_dir'}/".&tools::escape_chars($email);
     }
@@ -458,7 +462,7 @@ sub smime_encrypt {
 	my $temporary_file = $Conf{'tmpdir'}."/".$email.".".$$ ;
 
 	## encrypt the incomming message parse it.
-        do_log ('debug2', "xxxx $Conf{'openssl'} smime -encrypt -out $temporary_file -des3 $usercert");
+        do_log ('debug3', "xxxx $Conf{'openssl'} smime -encrypt -out $temporary_file -des3 $usercert");
 	if (!open(MSGDUMP, "| $Conf{'openssl'} smime -encrypt -out $temporary_file -des3 $usercert")) {
 	    &do_log('info', 'Can\'t encrypt message for recipient %s', $email);
 	}
@@ -488,7 +492,7 @@ sub smime_encrypt {
 	}						    
 	close NEWMSG;
 
-unlink ($temporary_file) unless ($main::options{'debug'} || $main::options{'debug2'}) ;
+unlink ($temporary_file) unless ($main::options{'debug'}) ;
 
 	## foreach header defined in  the incomming message but undefined in the
         ## crypted message, add this header in the crypted form.
@@ -515,20 +519,20 @@ sub smime_decrypt {
     my $msg = shift;
     my $list = shift ; ## the recipient of the msg
     
-    &do_log('debug2', 'tools::smime_decrypt message msg from %s,%s',$msg->head->get('from'),$list);
+    &do_log('debug2', 'tools::smime_decrypt message msg from %s,%s',$msg->head->get('from'),$list->{'name'});
 
-    my $certfile = "$Conf{'home'}/$list/cert.pem" ;
+    my $certfile = $list->{'dir'}."/cert.pem" ;
     unless (-r $certfile){
 	do_log('err', "unable to decrypt message : cert missing  $certfile");
 	return undef;
     }
-    my $keyfile = "$Conf{'home'}/$list/private_key";
+    my $keyfile = $list->{'dir'}."/private_key";
 
     unless (open (MSGDUMP , "> $Conf{'tmpdir'}/MSG.$$")) {
 	&do_log('err', 'unable to open %s/MSG.%s',$Conf{'tmpdir'},$$);
 	return undef;
     }
-    my $temporary_file = $Conf{'tmpdir'}."/".$list.".".$$ ;
+    my $temporary_file = $Conf{'tmpdir'}."/".$list->{'name'}.".".$$ ;
     
     ## dump the incomming message.
     if (!open(MSGDUMP,"> $temporary_file")) {
@@ -551,7 +555,7 @@ sub smime_decrypt {
 	}
     }
 
-    open (NEWMSG, "$Conf{'openssl'} smime -decrypt -in $temporary_file -recip $certfile -inkey $keyfile $pass_option 2>&1 |");
+    open (NEWMSG, "$Conf{'openssl'} smime -decrypt -in $temporary_file -recip $certfile -inkey $keyfile $pass_option |");
 
     if ($Conf{'key_passwd'} ne '') {
 	unless (open (FIFO,"> $Conf{'tmpdir'}/pass.$$")) {
@@ -568,14 +572,22 @@ sub smime_decrypt {
 	$msg_as_string .= $_;
     }
 	
+    close NEWMSG ;
+    unlink ($temporary_file) unless ($main::options{'debug'}) ;
+    
+    ## Create a MIME object with decrypted message
     my $parser = new MIME::Parser;
     $parser->output_to_core(1);
     unless ($decryptedmsg = $parser->parse_data($msg_as_string)) {
 	do_log('notice', 'Unable to parse message');
 	return undef;
     }
-    close NEWMSG ;
-    unlink ($temporary_file) unless ($main::options{'debug'} || $main::options{'debug2'}) ;
+
+    ## Now remove headers from $msg_as_string
+    my @msg_tab = split(/\n/, $msg_as_string);
+    my $line;
+    do {$line = shift(@msg_tab)} while ($line !~ /^\s*$/);
+    $msg_as_string = join("\n", @msg_tab);
     
     ## foreach header defined in the incomming message but undefined in the
     ## decrypted message, add this header in the decrypted form.
@@ -604,6 +616,19 @@ sub as_singlepart {
     my ($msg, $preferred_type) = @_;
     my $done = 0;
     
+    # First, if the message has a type of multipart/alternative
+    # make this the main message so we can get at the sub parts
+    my @parts = $msg->parts();
+    foreach my $index (0..$#parts) {
+        if ($parts[$index]->effective_type() =~ /^multipart\/alternative/) {
+            ## Only keep the multipart/alternative part
+            $msg->parts([$parts[$index]]);
+            $msg->make_singlepart();
+            last;
+        }
+    }
+
+    # Now look for the preferred_type and if found, make this the main message
     my @parts = $msg->parts();
     foreach my $index (0..$#parts) {
 	if ($parts[$index]->effective_type() =~ /^$preferred_type$/) {
@@ -628,7 +653,7 @@ sub escape_chars {
     ## Escape chars
     ##  !"#$%&'()+,:;<=>?[] AND accented chars
     ## escape % first
-    foreach my $i (0x25,0x20..0x24,0x26..0x2c,0x3a..0x3f,0x5b,0x5d,0xc0..0xff) {
+    foreach my $i (0x25,0x20..0x24,0x26..0x2c,0x3a..0x3f,0x5b,0x5d,0xa0..0xff) {
 	next if ($i == $ord_except);
 	my $hex_i = sprintf "%lx", $i;
 	$s =~ s/\x$hex_i/%$hex_i/g;
@@ -643,7 +668,7 @@ sub unescape_chars {
     my $s = shift;
 
     $s =~ s/%a5/\//g;  ## Special traetment for '/'
-    foreach my $i (0x20..0x2c,0x3a..0x3f,0x5b,0x5d,0xc0..0xff) {
+    foreach my $i (0x20..0x2c,0x3a..0x3f,0x5b,0x5d,0xa0..0xff) {
 	my $hex_i = sprintf "%lx", $i;
 	my $hex_s = sprintf "%c", $i;
 	$s =~ s/%$hex_i/$hex_s/g;
@@ -691,6 +716,46 @@ sub ciphersaber_installed {
     }
 }
 
+# create a cipher
+sub cookie_changed {
+    my $current=shift;
+    my $changed = 1 ;
+    if (-f "$Conf{'etc'}/cookies.history") {
+	unless (open COOK, "$Conf{'etc'}/cookies.history") {
+	    do_log('err', "Unable to read $Conf{'etc'}/cookies.history") ;
+	    return undef ; 
+	}
+	my $oldcook = <COOK>;
+	close COOK;
+
+	my @cookies = split(/\s+/,$oldcook );
+	
+
+	if ($cookies[$#cookies] eq $current) {
+	    do_log('debug2', "cookie is stable") ;
+	    $changed = 0;
+#	}else{
+#	    push @cookies, $current ;
+#	    unless (open COOK, ">$Conf{'etc'}/cookies.history") {
+#		do_log('err', "Unable to create $Conf{'etc'}/cookies.history") ;
+#		return undef ; 
+#	    }
+#	    printf COOK "%s",join(" ",@cookies) ;
+#	    
+#	    close COOK;
+	}
+	return $changed ;
+    }else{
+	unless (open COOK, ">$Conf{'etc'}/cookies.history") {
+	    do_log('err', "Unable to create $Conf{'etc'}/cookies.history") ;
+	    return undef ; 
+	}
+	printf COOK "$current ";
+	close COOK;
+	return(0);
+    }
+}
+
 ## encrypt a password
 sub crypt_password {
     my $inpasswd = shift ;
@@ -764,6 +829,7 @@ sub split_mail {
     my $message = shift ; 
     my $pathname = shift ;
     my $dir = shift ;
+    &do_log('debug2', 'tools::split_mail()');
 
     my $head = $message->head ;
     my $body = $message->body ;
@@ -779,10 +845,10 @@ sub split_mail {
     else { 
 	    my $fileExt ;
 
-	    if ($head->mime_attr("content_type.name") =~ /\.(\S+)/) {
+	    if ($head->mime_attr("content_type.name") =~ /\.(\S+)\s*\"*$/) {
 		$fileExt = $1 ;
 	    }
-	    elsif ($head->recommended_filename =~ /\.(\S+)/) {
+	    elsif ($head->recommended_filename =~ /\.(\S+)\s*\"*$/) {
 		$fileExt = $1 ;
 	    }
 	    else {
@@ -796,11 +862,11 @@ sub split_mail {
 
 	    ## Store body in file 
 	    unless (open OFILE, ">$dir/$pathname.$fileExt") {
-		print STDERR "Unable to open $dir/$pathname.$fileExt\n" ;
+		&do_log('err', "Unable to open $dir/$pathname.$fileExt") ;
 		return undef ; 
 	    }
 	    
-	    if ($encoding =~ /^binary|7bit|8bit|base64|quoted-printable|x-uu|x-uuencode|x-gzip64$/ ) {
+	    if ($encoding =~ /^(binary|7bit|8bit|base64|quoted-printable|x-uu|x-uuencode|x-gzip64)$/ ) {
 		open TMP, ">$dir/$pathname.$fileExt.$encoding";
 		$message->print_body (\*TMP);
 		close TMP;
@@ -808,6 +874,10 @@ sub split_mail {
 		open BODY, "$dir/$pathname.$fileExt.$encoding";
 
 		my $decoder = new MIME::Decoder $encoding;
+		unless (defined $decoder) {
+		    &do_log('err', 'Cannot create decoder for %s', $encoding);
+		    return undef;
+		}
 		$decoder->decode(\*BODY, \*OFILE);
 		unlink "$dir/$pathname.$fileExt.$encoding";
 	    }else {
@@ -817,20 +887,21 @@ sub split_mail {
 	    printf "\t-------\t Create file %s\n", $pathname.'.'.$fileExt ;
 	    
 	    ## Delete files created twice or more (with Content-Type.name and Content-Disposition.filename)
-	    $message->purge ;
-    
-	
+	    $message->purge ;	
     }
+    return 1;
 }
 
 sub virus_infected {
     my $mail = shift ;
     my $file = shift ;
-    &do_log('debug2', 'virus_infected (%s)', $file);
-    
-    # there is no virus anywhere if there is no antivus tools installed
-    return 0 unless ($Conf{'antivirus_path'} );
+
+    &do_log('debug2', 'Scan virus in %s', $file);
     
+    unless ($Conf{'antivirus_path'} ) {
+        &do_log('debug2', 'Sympa not configured to scan virus in message');
+	return 0;
+    }
     my @name = split(/\//,$file);
     my $work_dir = $Conf{'tmpdir'}.'/antivirus';
     
@@ -849,7 +920,10 @@ sub virus_infected {
     #$mail->dump_skeleton;
 
     ## Call the procedure of spliting mail
-    &split_mail ($mail,'msg', $work_dir) ;
+    unless (&split_mail ($mail,'msg', $work_dir)) {
+	&do_log('err', 'Could not split mail %s', $mail);
+	return undef;
+    }
 
     my $virusfound; 
     my $error_msg;
@@ -865,7 +939,8 @@ sub virus_infected {
 		
 	while (<ANTIVIR>) {
 	    $result .= $_; chomp $result;
-	    if (/^\s*Found the\s+(.*)\s*virus.*$/i){
+	    if ((/^\s*Found the\s+(.*)\s*virus.*$/i) ||
+		(/^\s*Found application\s+(.*)\.\s*$/i)){
 		$virusfound = $1;
 	    }
 	}
@@ -947,7 +1022,7 @@ sub virus_infected {
         if (( $status == 3) and not($virusfound)) { 
             $virusfound = "unknown";
         }    
-    }elsif ("${Conf{'antivirus_path'}}" =~ /AvpLinux$/) {
+    }elsif ("${Conf{'antivirus_path'}}" =~ /kavscanner/) {
 
 	# impossible to look for viruses with no option set
 	unless ($Conf{'antivirus_args'}) {
@@ -961,7 +1036,7 @@ sub virus_infected {
 	    if (/infected:\s+(.*)/){
 		$virusfound = $1;
 	    }
-	    elsif (/suspicion:\s+(.*)/){
+	    elsif (/suspicion:\s+(.*)/i){
 		$virusfound = $1;
 	    }
 	}
@@ -980,12 +1055,8 @@ sub virus_infected {
         # impossible to look for viruses with no option set
         return 0 unless ($Conf{'antivirus_args'});
     
-        #do_log('err',"Running: $Conf{'antivirus_path'} $Conf{'antivirus_args'} $work_dir |") ;
-
         open (ANTIVIR,"$Conf{'antivirus_path'} $Conf{'antivirus_args'} $work_dir |") ;
 	
-	#do_log('err', "Ok, entering virus_infected(), sweep part... ($work_dir) --liuk");
-        
 	while (<ANTIVIR>) {
 	    if (/Virus\s+(.*)/) {
 		$virusfound = $1;
@@ -999,7 +1070,26 @@ sub virus_infected {
 	if (( $status == 3) and not($virusfound)) {
 	    $virusfound = "unknown";
 	}
-    }        
+
+	## Clam antivirus
+    }elsif ("${Conf{'antivirus_path'}}" =~ /\/clamscan$/) {
+	
+        open (ANTIVIR,"$Conf{'antivirus_path'} $Conf{'antivirus_args'} $work_dir |") ;
+	
+	while (<ANTIVIR>) {
+	    if (/^\S+:\s(.*)\sFOUND$/) {
+		$virusfound = $1;
+	    }
+	}       
+	close ANTIVIR;
+        
+	my $status = $?/256 ;
+        
+	## Clamscan status =1 (*256) => virus
+	if (( $status == 1) and not($virusfound)) {
+	    $virusfound = "unknown";
+	}
+    }         
 
     ## Error while running antivir, notify listmaster
     if ($error_msg) {
@@ -1010,7 +1100,7 @@ sub virus_infected {
     unless ($main::options{'debug'}) {
 	opendir (DIR, ${work_dir});
 	my @list = readdir(DIR);
-	close (DIR);
+	closedir (DIR);
         foreach (@list) {
 	    my $nbre = unlink ("$work_dir/$_")  ;
 	}
@@ -1053,8 +1143,10 @@ sub adate {
 sub epoch_conv {
 
     my $arg = $_[0]; # argument date to convert
-    my $time = $_[1]; # the epoch current date
- 
+    my $time = $_[1] || time; # the epoch current date
+
+    &do_log('debug4','tools::epoch_conv(%s, %d)', $arg, $time);
+
     my $result;
     
      # decomposition of the argument date
@@ -1095,12 +1187,12 @@ sub date_conv {
 	return $arg;
     }
 	
-    if ($arg =~ /^(\d\d\d\dy)(\d+m)?(\d+d)?(\d+h)?(\d+min)?$/) { # absolute date
+    if ($arg =~ /^(\d\d\d\dy)(\d+m)?(\d+d)?(\d+h)?(\d+min)?(\d+sec)?$/) { # absolute date
 
-	my @date = (0, "$5", "$4", "$3", "$2", "$1");
+	my @date = ("$6", "$5", "$4", "$3", "$2", "$1");
 	for (my $i = 0; $i < 6; $i++) {
 	    chop ($date[$i]);
-	    if ($i == 1) {chop ($date[$i]); chop ($date[$i]);}
+	    if (($i == 1) || ($i== 2)) {chop ($date[$i]); chop ($date[$i]);}
 	    $date[$i] = 0 unless ($date[$i]);
 	}
 	$date[3] = 1 if ($date[3] == 0);
@@ -1120,15 +1212,15 @@ sub duration_conv {
 
     return 0 unless $arg;
   
-    $arg =~ /(\d+y)?(\d+m)?(\d+w)?(\d+d)?(\d+h)?(\d+min)?$/i ;
-    my @date = ("$1", "$2", "$3", "$4", "$5", "$6");
-    for (my $i = 0; $i < 6; $i++) {
+    $arg =~ /(\d+y)?(\d+m)?(\d+w)?(\d+d)?(\d+h)?(\d+min)?(\d+sec)?$/i ;
+    my @date = ("$1", "$2", "$3", "$4", "$5", "$6", "$7");
+    for (my $i = 0; $i < 7; $i++) {
 	chop ($date[$i]);
-	if ($i == 5) {chop ($date[$i]); chop ($date[$i]);}
+	if (($i == 5) || ($i == 6)) {chop ($date[$i]); chop ($date[$i]);}
 	$date[$i] = 0 unless ($date[$i]);
     }
     
-    my $duration = 60*($date[5]+60*($date[4]+24*($date[3]+7*$date[2]+365*$date[0])));
+    my $duration = $date[6]+60*($date[5]+60*($date[4]+24*($date[3]+7*$date[2]+365*$date[0])));
 	
     # specific processing for the months because their duration varies
     my @months = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31,
@@ -1144,13 +1236,38 @@ sub duration_conv {
 ## Look for a file in the list > robot > server > default locations
 sub get_filename {
     my ($type, $name, $robot, $list) = @_;
-    &do_log('debug2','tools::get_filename(%s,%s,%s,%s)', $type, $name, $robot, $list->{'name'});
+    &do_log('debug3','tools::get_filename(%s,%s,%s,%s)', $type, $name, $robot, $list->{'name'});
 
     if ($type eq 'etc') {
-	my @try = ("$Conf{'etc'}/$robot".'/'.$name,
-		   $Conf{'etc'}.'/'.$name,
-		   '--ETCBINDIR--'.'/'.$name);
+	my (@try, $default_name);
+	
+	## template refers to a language
+	## => extend search to default tpls
+	if ($name =~ /^(\S+)\.(\S+)\.tpl$/) {
+	    $default_name = $1.'.tpl';
+	    
+	    @try = ("$Conf{'etc'}/$robot".'/'.$name,
+		    "$Conf{'etc'}/$robot".'/'.$default_name,
+		    $Conf{'etc'}.'/'.$name,
+		    $Conf{'etc'}.'/'.$default_name,
+		    '--ETCBINDIR--'.'/'.$name,
+		    '--ETCBINDIR--'.'/'.$default_name);
+	}else {
+	    @try = ("$Conf{'etc'}/$robot".'/'.$name,
+		    $Conf{'etc'}.'/'.$name,
+		    '--ETCBINDIR--'.'/'.$name);
+	}
 	if ($list) {
+	    ## Default tpl
+	    if ($default_name) {
+		## No 'templates' subdir in list directory
+		if ($default_name =~ /^templates\/(.*)$/) {
+		    unshift @try, $list->{'dir'}.'/'.$1;
+		}else {
+		    unshift @try, $list->{'dir'}.'/'.$default_name;
+		}
+	    }
+
 	    ## No 'templates' subdir in list directory
 	    if ($name =~ /^templates\/(.*)$/) {
 		unshift @try, $list->{'dir'}.'/'.$1;
@@ -1159,14 +1276,14 @@ sub get_filename {
 	    }
 	}	
 	foreach my $f (@try) {
-#	    &do_log('debug','NAME: %s ; DIR %s', $name, $dir);
+	    &do_log('debug3','get_filname : NAME: %s ; DIR %s', $name, $dir);
 	    if (-r $f) {
 		return $f;
 	    }
 	}
     }
     
-    &do_log('info','Cannot find %s', $name);
+    &do_log('debug3','tools::get_filename: Cannot find %s', $name);
     return undef;
 }
 
@@ -1174,7 +1291,7 @@ sub write_pid {
     my ($pidfile, $pid) = @_;
 
     my $uid = (getpwnam('--USER--'))[2];
-    my $gid = (getpwnam('--GROUP--'))[2];
+    my $gid = (getgrnam('--GROUP--'))[2];
 
     my $piddir = $pidfile;
     $piddir =~ s/\/[^\/]+$//;
@@ -1208,4 +1325,69 @@ sub write_pid {
     return 1;
 }
 
+sub get_message_id {
+    my $robot = shift;
+
+    my $id = sprintf '<sympa.%d.%d.%d@%s>', time, $$, int(rand(999)), $robot;
+
+    return $id;
+}
+
+## Clean email address
+sub clean_email {
+    my $email = shift;
+
+    ## Lower-case
+    $email = lc($email);
+
+    ## remove leading and trailing spaces
+    $email =~ s/^\s*//;
+    $email =~ s/\s*$//;
+
+    return $email;
+}
+
+sub get_dir_size {
+    my $dir =shift;
+    
+    my $size=0;
+
+    if (opendir(DIR, $dir)) {
+	foreach my $file ( sort grep (!/^\./,readdir(DIR))) {
+	    if (-d "$dir/$file") {
+		$size += get_dir_size("$dir/$file");
+	    }
+	    else{
+		my @info = stat "$dir/$file" ;
+		$size += $info[7];
+	    }
+	}
+        closedir DIR;
+    }
+
+    return $size;
+}
+
+## Basic check of an email address
+sub valid_email {
+    my $email = shift;
+    
+    $email =~ /^([\w\-\_\.\/\+\=\']+|\".*\")\@[\w\-]+(\.[\w\-]+)+$/;
+}
+
+## Return canonical email address (lower-cased + space cleanup)
+## It could also support alternate email
+sub get_canonical_email {
+    my $email = shift;
+
+    ## Remove leading and trailing white spaces
+    $email =~ s/^\s*(\S.*\S)\s*$/$1/;
+
+    ## Lower-case
+    $email = lc($email);
+
+    return $email;
+}
+
 1;
+
